var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Loggable } from "./log";
/* == BASE ABILITY MODEL == */
var BaseAbility = /** @class */ (function (_super) {
    __extends(BaseAbility, _super);
    function BaseAbility(key, value, extra) {
        var _this = _super.call(this, "Ability") || this;
        _this.value = 1;
        _this.extra = null;
        _this.key = key;
        if (value) {
            _this.value = value;
        }
        if (extra) {
            _this.extra = extra;
        }
        return _this;
    }
    BaseAbility.FromString = function (abilityStr) {
        var values = abilityStr.split(" ");
        var abilityObj = new BaseAbility(values[0]);
        if (values.length > 1) {
            abilityObj.value = parseInt(values[1]);
            if (values.length > 2) {
                abilityObj.extra = values[2];
            }
        }
        return abilityObj;
    };
    return BaseAbility;
}(Loggable));
export { BaseAbility };
/* == ABILITIES LIST == */
var AbilityList = /** @class */ (function (_super) {
    __extends(AbilityList, _super);
    function AbilityList(parent) {
        var _this = _super.call(this, "AbilityList") || this;
        _this.parent = null;
        _this._abilities = [];
        _this._abilityKeys = [];
        if (parent) {
            _this.parent = parent;
        }
        return _this;
    }
    /* ABILITY GETTERS */
    AbilityList.prototype.getAbility = function (key) {
        if (!this._abilityKeys.includes(key)) {
            return null;
        }
        var abilityNum = this._abilityKeys.indexOf(key);
        return this._abilities[abilityNum];
    };
    /* returns value int of the ability or 0 when not included */
    AbilityList.prototype.getValue = function (key) {
        if (!this.hasAbility(key)) {
            return 0;
        }
        return this.getAbility(key).value;
    };
    AbilityList.prototype.getExtra = function (key) {
        if (!this.hasAbility(key)) {
            return;
        }
        return this.getAbility(key).extra;
    };
    Object.defineProperty(AbilityList.prototype, "length", {
        get: function () { return this._abilities.length; },
        enumerable: false,
        configurable: true
    });
    /* ABILITY ADDING */
    AbilityList.prototype._addAbility = function (abilityObj) {
        this._abilityKeys.push(abilityObj.key);
        this._abilities.push(abilityObj);
        return abilityObj;
    };
    AbilityList.prototype.addAbility = function (key, value, extra) {
        var abilityObj = new BaseAbility(key, value, extra);
        return this._addAbility(abilityObj);
    };
    /* ABILITY CHECKS */
    AbilityList.prototype.hasAbility = function (key, minValue) {
        var ability = this.getAbility(key);
        // ability not in list?
        if (ability) {
            if (!minValue) {
                return true;
            }
            if (minValue && ability.value && ability.value >= minValue) {
                return true;
            }
        }
        return false;
    };
    /* ABILITY STATIC METHODS */
    AbilityList.FromString = function (abilitiesStr, setParent) {
        // create new AbilityList object
        var abilityList = new AbilityList();
        if (setParent) {
            abilityList.parent = setParent;
        }
        // return empty list when string is empty
        var processString = abilitiesStr && abilitiesStr.length > 0;
        if (!processString) {
            return abilityList;
        }
        // or add abilities from the comma seperated strings
        var splittedStrings = abilitiesStr.split(', ');
        for (var _i = 0, splittedStrings_1 = splittedStrings; _i < splittedStrings_1.length; _i++) {
            var abilityStr = splittedStrings_1[_i];
            abilityList._addAbility(BaseAbility.FromString(abilityStr));
        }
        return abilityList;
    };
    return AbilityList;
}(Loggable));
export { AbilityList };
/* == ABILITY CONSTANTS == */
export var ABILITY_REALM_BATTLE = 0;
export var ABILITY_REALM_BATTLE_STATIC = 1;
export var ABILITY_REALM_WORLD = 2;
export var ABILITY_REALM_WORLD_STATIC = 3;
//# sourceMappingURL=ability.js.map