var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Loggable } from "./log";
import { Properties } from "./properties";
/* == EVENT == */
var BaseEvent = /** @class */ (function () {
    function BaseEvent(id, data) {
        this.properties = new Properties();
        this.id = id;
        if (data) {
            this.properties = data;
        }
    }
    return BaseEvent;
}());
export { BaseEvent };
/* == LISTENER == */
var BaseEventListener = /** @class */ (function (_super) {
    __extends(BaseEventListener, _super);
    function BaseEventListener(logName, isPrioritized) {
        var _this = _super.call(this, logName) || this;
        // flags
        _this.isMuted = false;
        _this.isPrioritized = false;
        // optional properties
        if (isPrioritized && isPrioritized == true) {
            _this.isPrioritized = true;
        }
        // register to eventmanager
        if (!globalThis.EVENTS) {
            _this.error("cant create listener ".concat(logName, ": EventManager not started"));
        }
        else {
            globalThis.EVENTS.registerListener(_this, _this.isPrioritized);
        }
        return _this;
    }
    BaseEventListener.prototype.onEvent = function (ev) {
    };
    return BaseEventListener;
}(Loggable));
export { BaseEventListener };
/* == MANAGER == */
var EventManager = /** @class */ (function (_super) {
    __extends(EventManager, _super);
    function EventManager(debugMode) {
        var _this = _super.call(this, "EventManager") || this;
        _this.currentEvent = null;
        _this.eventHistory = [];
        // listener arrays
        // private listeners: Array<IEventListener> = [];
        _this.defaultListeners = [];
        _this.prioritizedListeners = [];
        _this.debugMode = false;
        if (debugMode) {
            _this.debugMode = true;
        }
        _this.log("created");
        return _this;
    }
    EventManager.prototype.registerListener = function (listener, prioritize) {
        if (prioritize) {
            this.prioritizedListeners.push(listener);
        }
        else {
            this.defaultListeners.push(listener);
        }
    };
    EventManager.prototype.fireEvent = function (ev, verbose) {
        this.currentEvent = ev;
        // fire onEvent for every listener
        for (var _i = 0, _a = this.listeners; _i < _a.length; _i++) {
            var listener = _a[_i];
            if (!listener) {
                return;
            }
            var showLog = this.debugMode || verbose;
            if (showLog) {
                this.log("firing ".concat(ev.id, " in ").concat(listener));
            }
            listener.onEvent(ev);
        }
        // add event to history and clear currentEvent
        this.eventHistory.push(ev);
        this.currentEvent = null;
    };
    Object.defineProperty(EventManager.prototype, "listeners", {
        /* GETTERS & SETTERS */
        get: function () {
            var allListeners = [];
            allListeners.concat(this.prioritizedListeners);
            allListeners.concat(this.defaultListeners);
            return allListeners;
        },
        enumerable: false,
        configurable: true
    });
    /* DEBUG FUNCTIONS */
    EventManager.prototype.toString = function () {
        return "<EventManager listeners=".concat(this.listeners.length, " currentEvent=").concat(this.currentEvent, ">");
    };
    return EventManager;
}(Loggable));
export { EventManager };
/* == CONSTANTS == */
export var EV_CAT_GAME_LOADING = 0;
export var EV_CAT_ENTITIES = 2;
//# sourceMappingURL=events.js.map