var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Loggable } from "./log";
import { DawnbringerPalette } from "./colors";
import { NumPropertyModifier } from "./property-modifier";
// world
import { WorldPos } from "../game/world/pos";
/* == ABSTRACT MODEL == */
var AbstractProperty = /** @class */ (function () {
    function AbstractProperty(id, value, color, canAddModifiers) {
        this.color = DawnbringerPalette.WHITE;
        // modifiers
        this.canAddModifiers = false;
        this.modifiers = [];
        this.id = id;
        this._value = value;
        if (color) {
            this.color = color;
        }
        if (canAddModifiers) {
            this.canAddModifiers = true;
        }
    }
    Object.defineProperty(AbstractProperty.prototype, "value", {
        get: function () {
            if (this.modifiers.length <= 0) {
                return this._value;
            }
        },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    AbstractProperty.prototype.addModifier = function (modifier) {
        if (this.canAddModifiers) {
            this.modifiers.push(modifier);
        }
    };
    return AbstractProperty;
}());
export { AbstractProperty };
/* == PROPERTY TYPES == */
/* == STRING PROPERTY == */
var StrProperty = /** @class */ (function (_super) {
    __extends(StrProperty, _super);
    function StrProperty(id, value, color) {
        var _this = _super.call(this, id, value, color) || this;
        _this._value = "";
        _this._value = value;
        return _this;
    }
    Object.defineProperty(StrProperty.prototype, "value", {
        get: function () { return this._value; },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    return StrProperty;
}(AbstractProperty));
export { StrProperty };
/* == NUMBER PROPERTY == */
var NumProperty = /** @class */ (function (_super) {
    __extends(NumProperty, _super);
    function NumProperty(id, value, color) {
        var _this = _super.call(this, id, value, color) || this;
        _this._value = -1;
        _this._value = value;
        return _this;
    }
    Object.defineProperty(NumProperty.prototype, "value", {
        // return total value with modifiers
        get: function () {
            var totalValue = this._value;
            for (var _i = 0, _a = this.modifiers; _i < _a.length; _i++) {
                var modifier = _a[_i];
                if (modifier instanceof NumPropertyModifier) {
                    continue;
                }
                totalValue += modifier.absAmount;
            }
            return totalValue;
        },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    return NumProperty;
}(AbstractProperty));
export { NumProperty };
/* == BOOLEAN PROPERTY == */
var BoolProperty = /** @class */ (function (_super) {
    __extends(BoolProperty, _super);
    function BoolProperty(id, value, color) {
        var _this = _super.call(this, id, value, color) || this;
        _this._value = false;
        _this._value = value;
        return _this;
    }
    Object.defineProperty(BoolProperty.prototype, "value", {
        get: function () { return this._value; },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    return BoolProperty;
}(AbstractProperty));
export { BoolProperty };
/* == WORLD POSITION PROPERTY == */
var WorldPosProperty = /** @class */ (function (_super) {
    __extends(WorldPosProperty, _super);
    function WorldPosProperty(id, value, color) {
        var _this = _super.call(this, id, value, color) || this;
        _this._value = new WorldPos(0, 0);
        _this._value = value;
        return _this;
    }
    Object.defineProperty(WorldPosProperty.prototype, "value", {
        get: function () { return this._value; },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    return WorldPosProperty;
}(AbstractProperty));
export { WorldPosProperty };
/* == STRING ARRAY PROPERTY == */
var StrArrayProperty = /** @class */ (function (_super) {
    __extends(StrArrayProperty, _super);
    function StrArrayProperty(id, value, color) {
        var _this = _super.call(this, id, value, color) || this;
        _this._value = [];
        _this._value = value;
        return _this;
    }
    Object.defineProperty(StrArrayProperty.prototype, "value", {
        get: function () { return this._value; },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    return StrArrayProperty;
}(AbstractProperty));
export { StrArrayProperty };
/* == NUMBER ARRAY PROPERTY */
var NumArrayProperty = /** @class */ (function (_super) {
    __extends(NumArrayProperty, _super);
    function NumArrayProperty(id, value, color) {
        var _this = _super.call(this, id, value, color) || this;
        _this._value = [];
        _this._value = value;
        return _this;
    }
    Object.defineProperty(NumArrayProperty.prototype, "value", {
        get: function () { return this._value; },
        set: function (value) { this._value = value; },
        enumerable: false,
        configurable: true
    });
    return NumArrayProperty;
}(AbstractProperty));
export { NumArrayProperty };
/* == PROPERTY LIST == */
var Properties = /** @class */ (function (_super) {
    __extends(Properties, _super);
    function Properties(name) {
        var _this = _super.call(this, "Properties") || this;
        _this.name = null;
        _this.props = [];
        _this.ids = [];
        if (name) {
            _this.name = name;
        }
        return _this;
    }
    Properties.prototype.get = function (id) {
        for (var _i = 0, _a = this.props; _i < _a.length; _i++) {
            var prop = _a[_i];
            if (prop.id === id) {
                return prop._value;
            }
        }
        console.error('no property named "' + id + '"');
        return null;
    };
    Properties.prototype.getProperty = function (id) {
        for (var _i = 0, _a = this.props; _i < _a.length; _i++) {
            var prop = _a[_i];
            if (prop.id == id) {
                return prop;
            }
        }
        // this.error(`cant get property "${id}"`);
        return null;
    };
    Properties.prototype.add = function (prop) {
        this.ids.push(prop.id);
        this.props.push(prop);
    };
    Properties.prototype.addString = function (id, value, color) {
        this.add(new StrProperty(id, value, color));
    };
    Properties.prototype.addInteger = function (id, value, color, randomTo) {
        // generate random?
        if (randomTo && randomTo > 1) {
            value = Math.floor(Math.random() * randomTo);
        }
        this.add(new NumProperty(id, value, color));
    };
    Properties.prototype.addFloat = function (id, value, color) {
        this.add(new NumProperty(id, value, color));
    };
    Properties.prototype.addBoolean = function (id, value, color) {
        this.add(new BoolProperty(id, value, color));
    };
    Properties.prototype.addWorldPos = function (id, value, color) {
        this.add(new WorldPosProperty(id, value, color));
    };
    Properties.prototype.addStrArray = function (id, value, color) {
        this.add(new StrArrayProperty(id, value, color));
    };
    Properties.prototype.addNumArray = function (id, value, color) {
        this.add(new NumArrayProperty(id, value, color));
    };
    // getters
    Properties.prototype.getStr = function (id) { return this.get(id); };
    Properties.prototype.getNum = function (id) { return this.get(id); };
    Properties.prototype.getBool = function (id) { return this.get(id); };
    Properties.prototype.getWorldPos = function (id) { return this.get(id); };
    Properties.prototype.getStrArray = function (id) { return this.get(id); };
    Properties.prototype.getNumArray = function (id) { return this.get(id); };
    return Properties;
}(Loggable));
export { Properties };
//# sourceMappingURL=properties.js.map