var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { Loggable } from '../../../common/log';
// buildings
import { CATEGORIES, BuildingCategory } from './building-category';
/* == BUILDING TYPES == */
var AbstractBuildingType = /** @class */ (function (_super) {
    __extends(AbstractBuildingType, _super);
    function AbstractBuildingType(id, names) {
        var _this = _super.call(this, "BldType") || this;
        _this.id = id;
        _this.maxLevel = names.length + 1;
        _this.names = names;
        return _this;
    }
    AbstractBuildingType.prototype.onTick = function () {
    };
    Object.defineProperty(AbstractBuildingType.prototype, "name", {
        get: function () {
            return this.names[0];
        },
        enumerable: false,
        configurable: true
    });
    return AbstractBuildingType;
}(Loggable));
export { AbstractBuildingType };
var CivicBuildingType = /** @class */ (function (_super) {
    __extends(CivicBuildingType, _super);
    function CivicBuildingType(id, names) {
        var _this = _super.call(this, id, names) || this;
        _this.category = CATEGORIES[BuildingCategory.CIVIC];
        return _this;
    }
    return CivicBuildingType;
}(AbstractBuildingType));
export { CivicBuildingType };
var ProductionBuildingType = /** @class */ (function (_super) {
    __extends(ProductionBuildingType, _super);
    function ProductionBuildingType(resId) {
        var _this = _super.call(this, "prod-".concat(resId), []) || this;
        _this.category = CATEGORIES[BuildingCategory.PROD];
        _this.maxLevel = 10;
        var tradeRes = globalThis.RESOURCE_TYPES.get(resId);
        var prodBldName = tradeRes.prodBldName;
        _this.names = ["".concat(tradeRes.name, " ").concat(prodBldName)];
        return _this;
    }
    ProductionBuildingType.prototype.onTick = function () {
        _super.prototype.onTick.call(this);
    };
    Object.defineProperty(ProductionBuildingType.prototype, "name", {
        get: function () {
            return this.names[0];
        },
        enumerable: false,
        configurable: true
    });
    ProductionBuildingType.ResFromTypeId = function (bldTypeId) {
        if (!globalThis.TRADE) {
            console.warn("<BldType> cant get res type from bld id \"".concat(bldTypeId, "\""));
            return;
        }
        var resId = bldTypeId.replace("prod-", "");
        return globalThis.TRADE.getRes(resId);
    };
    return ProductionBuildingType;
}(AbstractBuildingType));
export { ProductionBuildingType };
/* BLD LISTS */
var CivicBuildingTypes = /** @class */ (function () {
    function CivicBuildingTypes() {
    }
    Object.defineProperty(CivicBuildingTypes, "CHURCH", {
        get: function () {
            return new CivicBuildingType("church", ["Chapel", "Abbey", "Church", "Large Church", "Catheral"]);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CivicBuildingTypes, "TOWNHALL", {
        get: function () {
            return new CivicBuildingType("townhall", ["Gathering Spot", "Mayor's Hut", "Townhall", "City Hall", "Palace"]);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CivicBuildingTypes, "MARKET", {
        get: function () {
            return new CivicBuildingType("market", ["Market Spot", "Local Market", "City Market", "Central Market"]);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CivicBuildingTypes, "GUILDHALL", {
        get: function () {
            return new CivicBuildingType("guildhall", ["Guild Meeting Room", "Guild House", "Guild Hall"]);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CivicBuildingTypes, "PALACE", {
        get: function () {
            return new CivicBuildingType("palace", ["Mayor Hut", "Town Manor", "Lord\'s Residence", "Great Palace"]);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CivicBuildingTypes, "IRRIGATION", {
        get: function () {
            return new CivicBuildingType("irrigation", ["Village Pond", "Field Canals", "Underground Irrigation"]);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CivicBuildingTypes, "LIST", {
        get: function () {
            return [
                CivicBuildingTypes.CHURCH,
                CivicBuildingTypes.TOWNHALL,
                CivicBuildingTypes.MARKET,
                CivicBuildingTypes.GUILDHALL,
                CivicBuildingTypes.PALACE,
                CivicBuildingTypes.IRRIGATION
            ];
        },
        enumerable: false,
        configurable: true
    });
    return CivicBuildingTypes;
}());
export { CivicBuildingTypes };
var ConstructableTypes = /** @class */ (function (_super) {
    __extends(ConstructableTypes, _super);
    function ConstructableTypes() {
        var _this = _super.call(this, "CityBuildingTypes") || this;
        _this._bldTypeIds = [];
        _this._bldTypeCategories = [];
        _this.bldTypes = [];
        return _this;
    }
    ConstructableTypes.prototype.add = function (bldType) {
        this.bldTypes.push(bldType);
        // add ids and categories into private arrays
        this._bldTypeIds.push(bldType.id);
        this._bldTypeCategories.push(bldType.category.num);
    };
    ConstructableTypes.prototype.get = function (bldTypeId) {
        if (!this._bldTypeIds.includes(bldTypeId)) {
            this.warn("cant get building of type \"".concat(bldTypeId, "\"; returns null"));
            return;
        }
        var bldTypeIndex = this._bldTypeIds.indexOf(bldTypeId);
        return this.bldTypes[bldTypeIndex];
    };
    Object.defineProperty(ConstructableTypes.prototype, "count", {
        /* CUSTOM BLD TYPE GETTERS */
        get: function () {
            return this._bldTypeIds.length;
        },
        enumerable: false,
        configurable: true
    });
    ConstructableTypes.prototype.getAll = function () {
        return this.bldTypes;
    };
    ConstructableTypes.prototype.getByCategory = function (bldCatNum) {
        var bldTypesOfCat = [];
        for (var bldTypeNum = 0; bldTypeNum < this.count; bldTypeNum++) {
            var bldCat = this._bldTypeCategories[bldTypeNum];
            if (bldCat == bldCatNum) {
                var bldTypeObj = this.bldTypes[bldTypeNum];
                bldTypesOfCat.push(bldTypeObj);
            }
        }
        return bldTypesOfCat;
    };
    return ConstructableTypes;
}(Loggable));
export { ConstructableTypes };
//# sourceMappingURL=building-type.js.map