var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { DawnbringerPalette } from "../../../common/colors";
import { RandomElement, RandomDice, RandomName } from "../../../helpers/random";
import * as EV from "../../game-events";
// world
import { WorldPos } from "../../world/pos";
// entities
import { TradeLocation } from "../../world/location";
// entities: city
import { BuildingCategory } from "../building/building-category";
import { CityPopulation, POPS_UPDATE_GROWTH } from "./population";
import { POP_PEASANTS_NUM } from './population-class';
import { CityCulture, CityCultureKeys } from "./city-culture";
// trade
import { TradeNode } from "../trade/trade-node";
import { MarketTrade } from "../trade/market-trade";
import { StoredResource } from "../trade/resource-storage";
// gui
import { CityDialog } from "../../gui/dialogs/city-dialog";
/* CITY INSTANCE */
var City = /** @class */ (function (_super) {
    __extends(City, _super);
    function City(num, name, pos, cityCultureKey, primaryResource, empireKey, featureStr) {
        var _this = 
        //super(`city-${num}`);
        _super.call(this, num, pos, name) || this;
        _this.taxesLastTurn = 0;
        // optional properties
        _this.owner = null;
        _this._cultureKey = "weur";
        _this.featureStr = null;
        _this.isColony = false;
        // factions & characters
        _this.leader = null;
        _this.localCharacters = [];
        // roads
        _this.roads = [];
        // trade
        _this.tradeNode = new TradeNode(999, false, _this);
        // units
        _this.militia = null;
        _this.patrols = [];
        // mesh
        _this.mesh = null;
        // city culture
        _this._cultureKey = cityCultureKey;
        _this.cityCulture = new CityCulture(_this, cityCultureKey);
        // optional city properties
        if (empireKey) {
            _this._factionKey = empireKey;
        }
        if (featureStr) {
            _this.featureStr = featureStr;
        }
        // city population
        _this.population = new CityPopulation(_this);
        // city properties
        _this.setRndStartingProperties();
        _this.primaryResource = primaryResource;
        return _this;
    }
    City.randomName = function () {
        return RandomName.from(CITY_NAME_PREFIXES, CITY_NAME_POSTFIXES);
    };
    City.prototype.addStartingBuildings = function () {
        // initialize resource production
        this._generateProductionBuildings();
        // give a random civic building
        this._generateStartingCivicBuildings();
    };
    City.prototype.toString = function () {
        var factionNum = this._ownerFactionNum;
        var posStr = "".concat(this.pos.x, " ").concat(this.pos.z);
        var ownerStr = "";
        if (globalThis.FACTIONS && factionNum) {
            var ownerFactionName = globalThis.FACTIONS.getFactionByNum(factionNum);
            ownerStr = " owner=\"".concat(ownerFactionName, "\"");
        }
        return "<City num=".concat(this.num, " name=\"").concat(this.name, "\" owner=").concat(ownerStr, " pos=\"").concat(posStr, ">\"");
    };
    /* FACTION */
    City.prototype.getOwner = function () {
        if (!this._ownerFactionNum) {
            this.log("cant get owner");
        }
        else {
            var ownerFaction = globalThis.FACTIONS.getFactionByNum(this._ownerFactionNum);
            return ownerFaction;
        }
    };
    City.prototype.getOwnerFactionNum = function (empireId) {
        var empireObj = globalThis.FACTIONS.getEmpireByKey(empireId);
        if (empireObj) {
            return empireObj.num;
        }
        return null;
    };
    /* POPULATION */
    City.prototype.increaseCityPops = function () {
        this.growth = 0;
        this.population.increaseRandom();
        this.log("City population increased");
    };
    City.prototype.onGrowthIncrease = function (growth) {
        if (this.growth >= POPS_UPDATE_GROWTH) {
            this.increaseCityPops();
        }
    };
    Object.defineProperty(City.prototype, "language", {
        get: function () {
            return this.cityCulture.languageId;
        },
        enumerable: false,
        configurable: true
    });
    /* TRADE */
    City.prototype.makeBestDelivery = function () {
        // this.log(`starting to make best delivery ..`);
        var profitableDeliveries = this.getProfitableDeliveries();
        var resAmount = 3;
        if (profitableDeliveries) {
            var rndDelivery = RandomElement.of(profitableDeliveries);
            // delivery is invalid
            if (!rndDelivery) {
                this.log("cant make delivery; no priftable delivery found in:");
                this.log(profitableDeliveries.toString());
                return false;
            }
            // city cant afford buying
            if (!this.canAfford(rndDelivery.resId, resAmount)) {
                var totalPrice = this.tradeNode.priceOf(rndDelivery.resId) * resAmount;
                this.log("cant deliver ".concat(rndDelivery.resId, " to ").concat(rndDelivery.target.name, "; city cant afford ").concat(totalPrice, " coins"));
                return false;
            }
            this.deliverResources(rndDelivery.target, rndDelivery.resId);
            return true;
        }
        this.log("cant find a profitable delivery in ".concat(this.connections.length, " connected cities"));
        return false;
    };
    City.prototype.deliverResources = function (target, resId, amount) {
        if (!amount) {
            amount = 3;
        }
        var targetMarket = target.tradeNode;
        // calculate cities profit of trading resources
        var price = this.tradeNode.priceOf(resId);
        var targetPrice = targetMarket.priceOf(resId);
        var profit = amount * (targetPrice - price);
        // exchange money from trading target to delivering city
        target.money -= amount * targetPrice;
        this.money += profit;
        // exchange resources
        this.tradeNode.remove(resId, amount);
        targetMarket.add(resId, amount);
        // log & inform managers
        this.log("delivered ".concat(amount, " ").concat(resId, " for ").concat(profit, " coins profit to ").concat(target.name));
        var storedRes = new StoredResource(resId, amount, price, profit);
        globalThis.TRADE.onDelivery(this, target, storedRes);
    };
    /* BUILDINGS */
    City.prototype._generateProductionBuildings = function () {
        // primary production
        var bldTypeId = "prod-".concat(this.primaryResource);
        this.addBuilding(bldTypeId, 2);
        // secondary production
        for (var secProductions = 0; secProductions < 2; secProductions++) {
            var rndRes = RandomElement.of(RESOURCE_TYPES.getIds());
            this.addBuilding("prod-".concat(rndRes), 1);
        }
    };
    City.prototype._generateStartingCivicBuildings = function () {
        var startingCivicBlds = 1;
        if (this.featureStr == CITY_FEATURE_DEVELOPED) {
            startingCivicBlds += 1;
        }
        for (var index = 0; index < startingCivicBlds; index++) {
            this.buildRandomCivicBuilding();
        }
    };
    City.prototype.buildRandomCivicBuilding = function () {
        var civicBldTypes = globalThis.CITIES.getBuildingTypesByCat(BuildingCategory.CIVIC);
        var randomType = RandomElement.of(civicBldTypes);
        this.addBuilding(randomType.id);
    };
    City.prototype.setRndStartingProperties = function () {
        this.properties.addInteger(CITY_PROP_GROWTH, -1, DawnbringerPalette.CHRISTI, 10);
        this.properties.addInteger(CITY_PROP_MIGHT, 0, DawnbringerPalette.MANDY);
        this.properties.addInteger(CITY_PROP_HAPPINESS, -1, DawnbringerPalette.CLAIRVOYANT, 10);
        this.properties.addInteger(CITY_PROP_FAITH, -1, DawnbringerPalette.CORNFLOWER, 6);
    };
    /* CITY FEAUTURES */
    City.prototype.loadFeatures = function () {
        if (!this.featureStr) {
            return;
        }
        // additional connections
        if (this.featureStr.startsWith("conn")) {
            var targetCityKey = this.featureStr.replace("conn ", "");
            var targetCity = globalThis.CITIES.getByKey(targetCityKey);
            this.addConnectionTo(targetCity, true);
            this.log("additional connection between ".concat(this.name, " and ").concat(targetCity.name, " created"));
        }
    };
    /* RESOURCE PRODUCTION & CONSUMPTION */
    City.prototype.getProducableResources = function () {
        var terrainResTypes = [];
        for (var _i = 0, _a = globalThis.RESOURCE_TYPES.getAll(); _i < _a.length; _i++) {
            var resObj = _a[_i];
            if (this.canProduce(resObj.id)) {
                terrainResTypes.push(resObj);
            }
        }
        return terrainResTypes;
    };
    City.prototype._processProduction = function () {
        for (var _i = 0, _a = this.buildings; _i < _a.length; _i++) {
            var bld = _a[_i];
            if (!bld.bldType) {
                this.log("cant produce resources in ".concat(bld.name, "; no bldtype of building"));
                continue;
            }
            if (bld.bldType.category.isProd) {
                var resId = bld.bldType.id.replace("prod-", "");
                var amount = bld.level * 0.2 * 2;
                if (this.tradeNode.hasRequirementsFor(resId, amount)) {
                    this.produceResource(resId, amount);
                }
                else {
                    var resName = globalThis.RESOURCE_TYPES.get(resId).name;
                    // this.warn(`cant produce ${amount} ${resName}; requirements not met`);
                }
            }
        }
    };
    City.prototype._collectTaxes = function () {
        var income = this.population.getTotalTaxes();
        this.money += income;
        this.taxesLastTurn = income;
    };
    /* returns an Array of TradeResourceProfit Models with connected towns when no losses are made */
    City.prototype.getProfitableDeliveries = function () {
        var profitable = [];
        for (var _i = 0, _a = this.getStoredResources(); _i < _a.length; _i++) {
            var storedRes = _a[_i];
            var resId = storedRes.resId;
            var connectedTradeLocations = this.getConnectedTradeLocations();
            if (connectedTradeLocations.length <= 0) {
                this.error("cant find most profitable delivery; no connected trade locations");
            }
            for (var _b = 0, connectedTradeLocations_1 = connectedTradeLocations; _b < connectedTradeLocations_1.length; _b++) {
                var connected = connectedTradeLocations_1[_b];
                var profit = connected.tradeNode.priceOf(resId) - this.tradeNode.priceOf(resId);
                if (profit > 0) {
                    var turnNum = globalThis.TURNS.turnNum();
                    var profitObj = new MarketTrade(turnNum, resId, profit, connected, this);
                    profitable.push(profitObj);
                }
            }
        }
        return profitable;
    };
    /* ASSETS */
    City.prototype.getAssetName = function () {
        if (!this._cultureKey) {
            return CITY_MODEL_DEFAULT;
        }
        return "city_".concat(this._cultureKey);
    };
    /* CONNECTIONS */
    City.prototype.addConnections = function (maxDist) {
        var connections = [];
        // city connection models
        for (var _i = 0, _a = globalThis.CITIES.getAll(); _i < _a.length; _i++) {
            var otherCity = _a[_i];
            if (!this.equals(otherCity)) {
                var distance = this.pos.distanceTo(otherCity.pos);
                if (distance <= maxDist) {
                    var conn = this.addTradeConnectionTo(otherCity, false);
                    connections.push(conn);
                }
            }
        }
        // find simple path
        for (var _b = 0, _c = this.connections; _b < _c.length; _b++) {
            var conn = _c[_b];
            // connection starts here?
            if (conn.source.num == this.num) {
                conn.path.setDirectPath();
            }
        }
        // return created connection
        return connections;
    };
    /* ROADS */
    City.prototype.getRoads = function () {
        return this.roads;
    };
    Object.defineProperty(City.prototype, "meshId", {
        /* WORLD */
        get: function () {
            return "city-" + this.key + "-mesh";
        },
        enumerable: false,
        configurable: true
    });
    City.GetKeyByMeshName = function (cityMeshName) {
        var cityKey = cityMeshName.split("-")[1];
        return cityKey;
    };
    City.prototype.hideIdTooFar = function () {
        if (!this.mesh) {
            return;
        }
        var playerPos = globalThis.PLAYER.pos;
        var distToPlayer = this.pos.distanceTo(playerPos);
        if (distToPlayer > CITY_RENDER_MAX_DIST) {
            // this.log(`hiding city ${this.name}: ${distToPlayer} is too far`);
            this.mesh.setEnabled(false);
        }
        else {
            this.mesh.setEnabled(true);
        }
    };
    /* EVENT LISTENING */
    City.prototype.onEvent = function (ev) {
        // no event listening before game start
        if (!globalThis.GAME_HAS_STARTED) {
            return;
        }
        else if (ev.id == EV.ID_CITY_ENTERED) {
            var enteredCityNum = ev.properties.getNum("num");
            if (enteredCityNum == this.num) {
                this.onEnter(globalThis.PLAYER);
            }
        }
        else if (ev.id = EV.ID_TURN_STARTED) {
            this.hideIdTooFar();
        }
    };
    City.prototype.onEnter = function (playerOrUnit) {
        this.log("is entered by ".concat(playerOrUnit));
        var diag = new CityDialog(this);
        globalThis.GUI.showDialog(diag);
    };
    City.prototype.onLeave = function (playerOrUnit) {
        this.log("is left by player");
    };
    City.prototype.onTurn = function () {
        _super.prototype.onTurn.call(this);
        this._collectTaxes();
        this._processProduction();
        // this._processConsumption();
        // pop consumption
        if (globalThis.TURNS.turnNum() % 7 == 0) {
            var successfullPercent = this.population.consumeResources() * 100;
            // this.log(`pop consumption: ${successfullPercent}%`);
        }
        // form caravan?
        var formCaravan = RandomDice.roll(40);
        if (formCaravan) {
            this.makeBestDelivery();
        }
    };
    /* LOAD OR GENERATE */
    City.FromValueArray = function (cityValueArray) {
        var name = cityValueArray[1];
        var pos = WorldPos.FromString(cityValueArray[2]);
        var culture = cityValueArray[3];
        var empireKey = cityValueArray[4];
        var primRes = cityValueArray[5];
        // city features?
        var featuresStr = null;
        if (cityValueArray.length > 6) {
            featuresStr = cityValueArray[6];
        }
        var cityIsColony = cityValueArray.length > 7 && cityValueArray[7] == "true";
        // create city object
        var cityNum = globalThis.CITIES.getAll().length;
        var newCity = new City(cityNum, name, pos, culture, primRes, empireKey);
        newCity.featureStr = featuresStr;
        // including city key ..
        var key = cityValueArray[0];
        newCity.key = key;
        // .. and possibly as colony
        if (cityIsColony) {
            newCity.isColony = true;
        }
        // return city with key and colony flag
        return newCity;
    };
    City.MakeRandom = function () {
        // let cityNum: number = globalThis.CITIES.getAll().length;
        var rndName = City.randomName();
        var rndPos = WorldPos.random();
        var rndCulture = CityCultureKeys.GetRandom();
        var rndPrimRes = RandomElement.of(globalThis.RESOURCE_TYPES.getIds());
        return new City(null, rndName, rndPos, rndCulture, rndPrimRes);
    };
    Object.defineProperty(City.prototype, "growth", {
        /* GETTERS & SETTERS */
        // growth
        get: function () {
            var baseGrowth = this.properties.getNum(CITY_PROP_GROWTH);
            // add 0.5 growth per peasant
            var peasantPopsAmount = this.population.getPopsOf(POP_PEASANTS_NUM);
            var peasantGrowth = peasantPopsAmount * 0.25;
            return baseGrowth + peasantGrowth;
        },
        set: function (value) {
            this.properties.getProperty(CITY_PROP_GROWTH).value = value;
            this.onGrowthIncrease(value);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(City.prototype, "might", {
        // property getters
        get: function () { return this.properties.getNum(CITY_PROP_MIGHT); },
        // property setters
        set: function (value) { this.properties.getProperty(CITY_PROP_MIGHT).value = value; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(City.prototype, "happiness", {
        get: function () { return this.properties.getNum(CITY_PROP_HAPPINESS); },
        set: function (value) { this.properties.getProperty(CITY_PROP_HAPPINESS).value = value; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(City.prototype, "faith", {
        get: function () { return this.properties.getNum(CITY_PROP_FAITH); },
        set: function (value) { this.properties.getProperty(CITY_PROP_FAITH).value = value; },
        enumerable: false,
        configurable: true
    });
    return City;
}(TradeLocation));
export { City };
/* == CITY PROPERTIES == */
export var CITY_PROP_GROWTH = "growth";
export var CITY_PROP_MIGHT = "might";
export var CITY_PROP_HAPPINESS = "happiness";
export var CITY_PROP_FAITH = "faith";
/* == CITY ASSETS == */
export var CITY_MODEL_DEFAULT = "rural_townhall";
/* == CITY CONSTANTS == */
export var CITY_CONNECTION_MAX_DIST = 10;
export var CITY_RENDER_MAX_DIST = 35;
// random names
export var CITY_NAME_PREFIXES = ["Santa", "Port", "New", "", "Cap", "Fort", "Old", "Point"];
export var CITY_NAME_POSTFIXES = [" Royale", " London", " Marino", " Brugges", " Mecca", " Marble", "green", "Cross", "cross", "rock", "stadt"];
// city features
export var CITY_FEATURE_DEVELOPED = "developed";
export var CITY_FEATURE_INDUSTRIAL = "industrial";
export var CITY_FEATURE_CONN_PREFIX = "conn ";
//# sourceMappingURL=city.js.map