var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { RandomWorldPos } from "../../../helpers/random";
import { WorldEventEntity } from "../../../common/entity";
import { UnitArmy } from "../unit/army/army";
import { ArmyCategory } from "../unit/army/army-category";
// faction classes
import { FactionAi } from "./ai/faction-ai";
import { FactionDiplomacy } from "./diplomacy/diplomacy";
import { FactionFlag } from "./faction-flag";
import { FACTION_TYPES, TYPE_REBEL } from "./faction-type";
/* == FACTION MODEL == */
/* Base Faction which families and empires inherit from; owns towns and armies & can handle
   diplomatic deals with other factions */
var AbstractFaction = /** @class */ (function (_super) {
    __extends(AbstractFaction, _super);
    function AbstractFaction(key, name, home, flagStr) {
        var _this = _super.call(this, -1, null) || this;
        _this.factionType = FACTION_TYPES[TYPE_REBEL];
        _this.prestige = 0;
        // armies
        _this.factionArmies = [];
        // ai & diplomacy
        _this.ai = null;
        _this.key = key;
        // assign faction num
        if (!globalThis.FACTIONS) {
            _this.num = -1;
        }
        else {
            _this.num = globalThis.FACTIONS.nextFactionNum();
        }
        // set base properties
        _this.name = name;
        _this.flag = new FactionFlag(_this, flagStr);
        // optional properties
        if (home) {
            _this.home = home;
            _this.pos = _this.home.pos;
        }
        // diplomacy & ai
        _this.diplomat = new FactionDiplomacy(_this);
        _this.ai = new FactionAi(_this);
        return _this;
    }
    AbstractFaction.prototype.updateFlag = function () {
        this.flag = new FactionFlag(this, this.flagData);
    };
    AbstractFaction.prototype.createNewArmy = function (isPatrol) {
        if (!this.home) {
            this.error("cant create army; has no home");
            return;
        }
        var armyCatNum = ArmyCategory.FACTION_ARMY;
        if (isPatrol) {
            armyCatNum = ArmyCategory.FACTION_PATROL;
        }
        // find random position around faction home
        var centerPos = this.home.pos;
        var rndPos = RandomWorldPos.around(centerPos, ARMY_SPAWNING_RADIUS);
        // add new UnitArmy model
        var newArmy = new UnitArmy(armyCatNum, rndPos);
        newArmy.parentFaction = this;
        // add army to this factions army array
        this.factionArmies.push(newArmy);
    };
    /* sets home if first city & adds this city to ownedCity list */
    AbstractFaction.prototype.giveCityOwnership = function (ownedCity, beforeStart) {
        ownedCity.owner = this;
        var isFirstCity = this.ownedCities.length == 0;
        this.ownedCities.push(ownedCity);
        // when city is initial city (not conquered) -> set as home
        if (beforeStart && isFirstCity) {
            this.home = ownedCity;
        }
    };
    Object.defineProperty(AbstractFaction.prototype, "ownedCities", {
        /* FACTION GETTERS */
        get: function () {
            var factionAndArmyMgrLoaded = globalThis.CITIES && globalThis.FACTIONS;
            if (!factionAndArmyMgrLoaded) {
                this.warn("cant get owned Cities of \"".concat(this.name, "\": CityManager or FactionManager not found"));
                return;
            }
            return globalThis.CITIES.getCitiesByFactionKey(this.key);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(AbstractFaction.prototype, "attackArmies", {
        get: function () {
            var attackArmies = [];
            for (var _i = 0, _a = this.factionArmies; _i < _a.length; _i++) {
                var factionArmy = _a[_i];
                if (factionArmy.armyCat.num == ArmyCategory.FACTION_ARMY) {
                    attackArmies.push(factionArmy);
                }
            }
            return attackArmies;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(AbstractFaction.prototype, "patrolArmies", {
        get: function () {
            var patrolArmies = [];
            for (var _i = 0, _a = this.factionArmies; _i < _a.length; _i++) {
                var factionArmy = _a[_i];
                if (factionArmy.armyCat.num == ArmyCategory.FACTION_PATROL) {
                    patrolArmies.push(factionArmy);
                }
            }
            return patrolArmies;
        },
        enumerable: false,
        configurable: true
    });
    return AbstractFaction;
}(WorldEventEntity));
export { AbstractFaction };
/* == CONSTANTS == */
var ARMY_SPAWNING_RADIUS = 3;
//# sourceMappingURL=faction.js.map