var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { WorldEventEntity } from "../../../common/entity";
import * as EV from "../../game-events";
import { TerrainTypes } from "../../world/tile/tile-terrain";
import { WorldPos, HexDirection } from "../../world/pos";
import { WorldTools } from "../../../helpers/world-tools";
// babylon
import * as BABYLON from 'babylonjs';
var Player = /** @class */ (function (_super) {
    __extends(Player, _super);
    function Player() {
        var _this = _super.call(this, 0, null) || this;
        _this.num = -1;
        _this.pos = new WorldPos(100, 50);
        _this.coins = STARTING_MONEY;
        _this.family = null;
        _this.getFamily();
        _this.setStartPos();
        _this.showDummy();
        return _this;
    }
    // PLAYER START POSITION
    Player.prototype.setStartPos = function () {
        var mapStartPos = globalThis.MAP.playerStartingPos;
        this.log("setting starting pos selected from map at ".concat(mapStartPos.asString()));
        this.pos = mapStartPos;
        if (this.family && this.family.customStartPos) {
            this.pos = this.family.customStartPos;
        }
    };
    Player.prototype.setStartPosFromHome = function () {
        var playerHome = this.family.homeCity;
        if ((!this.family) || (!this.family.homeCity)) {
            this.error("cant get player starting pos; player family home is not set");
            return;
        }
        this.log("setting starting pos to ".concat(this.family.homeCity.name, " at ").concat(this.family.homeCity.pos.asString()));
        this.pos = this.family.homeCity.pos;
    };
    Player.prototype.getFamily = function () {
        if (!globalThis.FACTIONS) {
            this.error("cant get player family; FactionManager hasnt loaded yet");
            return;
        }
        this.family = globalThis.FACTIONS.playerFamily;
    };
    // PLAYER 3D MODEL
    /* shows a sphere on player position before models are loaded */
    Player.prototype.showDummy = function () {
        if (!globalThis.SCENE || !globalThis.WORLD) {
            this.error("cant create; scene or world isnt loaded yet");
        }
        var sphereOptions = {
            diameter: 2
        };
        // create sphere dummy mesh
        this.mesh = BABYLON.MeshBuilder.CreateSphere("player-mesh", sphereOptions, globalThis.SCENE);
        // with red standard mat
        var mat = new BABYLON.StandardMaterial("player-mat", globalThis.SCENE);
        mat.diffuseColor = new BABYLON.Color3(1, 0, 0);
        this.mesh.material = mat;
        // at player position
        var scenePos = this.scenePos;
        scenePos.y += .5;
        this.mesh.position = scenePos;
        // inform managers
        if (globalThis.SELECTION) {
            globalThis.SELECTION.setSourceToPlayer();
        }
    };
    /* updates player model to horseman on land or small ship on water */
    Player.prototype.updateMesh = function () {
        // save pos & remove old mesh
        var meshPos = this.mesh.position;
        this.mesh.dispose();
        var tile = globalThis.MAP.tileAtPos(this.pos);
        var isOnLand = tile.isOnLand;
        // get new mesh depending whether player is on land or water
        if (isOnLand) {
            this.mesh = globalThis.ASSETS.getAsset(ASSET_LAND).getMesh().clone("player", null);
        }
        else {
            this.mesh = globalThis.ASSETS.getAsset(ASSET_WATER).getMesh().clone("player", null);
        }
        this.mesh.position = meshPos;
        this.mesh.scaling = new BABYLON.Vector3(2, 2, 2);
        // set the camera target to newly created mesh
        globalThis.WORLD.renewCameraTarget(this.mesh);
        // inform managers
        if (globalThis.SELECTION) {
            globalThis.SELECTION.setSourceToPlayer();
        }
    };
    Player.prototype.setArmy = function (army) {
        this.army = army;
    };
    // PLAYER MOVEMENT
    Player.prototype.moveInDir = function (dirStr) {
        // 1. save latest position in constant
        var startPos = this.pos;
        // 2. calculate new relative and absolute target position
        var relPos = new HexDirection(dirStr).relPos;
        var targetPos = new WorldPos(this.pos.x + relPos.x, this.pos.z + relPos.z);
        this.setPos(targetPos, dirStr, startPos);
    };
    Player.prototype.setPos = function (targetPos, dirStr, startPos) {
        if (!globalThis.PLAYER_M) {
            return;
        }
        // inform managers
        globalThis.PLAYER_M.beforePlayerMovement(targetPos);
        // check possibility
        var canMove = globalThis.PLAYER_M.canMoveTo(targetPos);
        if (!canMove) {
            return;
        }
        globalThis.PLAYER_M.beforeValidPlayerMovement(targetPos);
        // 3. fire player moved event
        globalThis.EVENTS.fireEvent(new EV.PlayerMovedEvent(targetPos, dirStr, startPos));
        this.pos = targetPos;
        // inform managers
        globalThis.PLAYER_M.afterPlayerMovement();
    };
    /* appliest post movement changes: enters or leaves city? */
    Player.prototype.checkCityEnterLeave = function () {
        /* add after movement checks here */
        if (globalThis.GUI) {
            globalThis.GUI.onPlayerMoved();
        }
        // entered city?
        if (this.localCity) {
            // this.isInCity = cityAt;
            globalThis.EVENTS.fireEvent(new EV.CityEnteredEvent(this.localCity));
        }
        else if (this.localCity) {
            // todo fix leaving
            // player left city: tell city that player has left
            // this.isInCity.onLeave(this);
            //this.isInCity = null;
        }
    };
    Player.prototype.updatePlayerYPos = function () {
        this.mesh.position = this.scenePos;
        this.mesh.position.y += this.groundOffset;
    };
    Object.defineProperty(Player.prototype, "groundOffset", {
        /* returns height between world 0 and terrain top as number in world space */
        get: function () {
            var heightOffset = Player.HEIGHT_OFFSET;
            if (this.tile) {
                // terrain offset: half of tile height
                heightOffset += (TerrainTypes.heightOf(this.tile.terrainChar) / 2);
            }
            return heightOffset;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Player.prototype, "localCity", {
        get: function () {
            if (!globalThis.CITIES) {
                this.error("cant get local city if player; CityManager not found");
                return;
            }
            return globalThis.CITIES.AtPos(this.pos);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Player.prototype, "storage", {
        /* PLAYER STORAGE */
        get: function () {
            if (!this.army) {
                return null;
            }
            return this.army.storage;
        },
        enumerable: false,
        configurable: true
    });
    Player.prototype.getStoredRes = function (resId) {
        if (!this.storage) {
            return null;
        }
        var resInStorage = this.army.getStoredRes(resId);
        return resInStorage;
    };
    Player.prototype.amountInStorage = function (resId) {
        if (!this.storage) {
            return 0;
        }
        var fixedStr = this.storage.amountOf(resId).toFixed(1);
        return parseFloat(fixedStr);
    };
    /* EVENT HANDLING */
    Player.prototype.onEvent = function (ev) {
        if (ev.id == EV.ID_CITIES_PLACED) {
            // this.createInitialArmy();
        }
    };
    /* PLAYER GETTERS */
    Player.prototype.toString = function () {
        return "<Player ".concat(this.pos.x, "|").concat(this.pos.z, ">");
    };
    Object.defineProperty(Player.prototype, "tile", {
        get: function () {
            return globalThis.MAP.tileAtPos(this.pos);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Player.prototype, "scenePos", {
        get: function () {
            return WorldTools.worldToScenePos(this.pos);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Player, "HEIGHT_OFFSET", {
        get: function () { return 0; },
        enumerable: false,
        configurable: true
    });
    return Player;
}(WorldEventEntity));
export { Player };
/* == CONSTANTS == */
// default properties
export var STARTING_MONEY = 500;
// 3d properties
export var ASSET_WATER = "ship_smallest";
export var ASSET_LAND = "rider";
//# sourceMappingURL=player.js.map