// common
import { ColorPalette } from "../../../helpers/colorpalette";
// helpers
import { Strings } from "../../../helpers/strings";
var TradeResource = /** @class */ (function () {
    function TradeResource(id, name, color, defaultPrice, requires, prodBldType, bldTerrainStr, prodRequiredBld) {
        this.requires = [];
        this.prodBldType = PROD_BLD_DEFAULT;
        this.id = id;
        this.name = name;
        this.color = color;
        this.defaultPrice = defaultPrice;
        // optional properties
        if (requires && requires.length > 0) {
            this.requires = requires;
        }
        if (prodBldType) {
            this.prodBldType = prodBldType;
        }
        if (bldTerrainStr) {
            this.setProdBldTerrain(bldTerrainStr);
        }
        if (prodRequiredBld) {
            this._prodRequiredBldId = prodRequiredBld;
        }
    }
    TradeResource.FromJson = function (tradeResJson) {
        var resId = tradeResJson["id"];
        var resName = tradeResJson["name"];
        var colorName = tradeResJson["colorName"];
        var defaultPrice = tradeResJson["price"];
        var color = ColorPalette.NameAsNumArray(colorName);
        var requires = tradeResJson["requires"];
        var bldType = tradeResJson["bldType"];
        var tradeRes = new TradeResource(resId, resName, color, defaultPrice, requires, bldType);
        return tradeRes;
    };
    TradeResource.prototype.setProdBldTerrain = function (bldTerrainStr) {
        if (bldTerrainStr && bldTerrainStr.length > 0) {
            this._prodBldTerrainStr = bldTerrainStr;
            var terrainChars = bldTerrainStr.split(" ");
            this.restrictedProdBldTerrains = terrainChars;
        }
    };
    /* TRADE RES FLAG GETTERS */
    TradeResource.prototype.canBuildProdBldOn = function (terrainChar) {
        if (!this.hasProdBldTerrainRequirement) {
            return true;
        }
        if (this.restrictedProdBldTerrains.includes(terrainChar)) {
            return true;
        }
        return false;
    };
    Object.defineProperty(TradeResource.prototype, "hasResRequirement", {
        get: function () {
            return this.requires && this.requires.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TradeResource.prototype, "hasProdBldTerrainRequirement", {
        get: function () {
            return this._prodBldTerrainStr && this._prodBldTerrainStr.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TradeResource.prototype, "hasRequirement", {
        get: function () {
            return this.requires.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    TradeResource.prototype.equals = function (otherResource) {
        if (this.id == otherResource.id) {
            return true;
        }
        return false;
    };
    Object.defineProperty(TradeResource.prototype, "prodBldName", {
        /* returns the name of this resource's producing building as capitalized string */
        get: function () {
            return Strings.capitalizeFirst(this.prodBldType);
        },
        enumerable: false,
        configurable: true
    });
    return TradeResource;
}());
export { TradeResource };
export var PROD_BLD_DEFAULT = "hut";
//# sourceMappingURL=resource.js.map