var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as EV from "../../../game-events";
import { TradeNode } from "../../trade/trade-node";
// helpers
import { RandomElement } from "../../../../helpers/random";
import { OwnedLocation } from '../../../world/location';
import { ArmyCategory } from './army-category';
/* == ARMY MODEL == */
var UnitArmy = /** @class */ (function (_super) {
    __extends(UnitArmy, _super);
    function UnitArmy(armyCatNum, pos, overwriteStorageSpace, targetPos) {
        var _this = _super.call(this, -1, pos, "Unnamed Army", null) || this;
        // flags
        _this.isFortified = false;
        _this.parentIsPlayer = false;
        // unit list
        _this.units = [];
        // storage
        _this.storageSpace = 4;
        _this.storedResources = [];
        _this.armyCat = new ArmyCategory(armyCatNum);
        // optional parameters
        if (overwriteStorageSpace) {
            _this.storageSpace = overwriteStorageSpace;
        }
        _this.storage = new TradeNode(_this.storageSpace, true, _this);
        // give random name
        _this.name = _this.getRandomName();
        // fire ArmySpawnedEvent
        var gameStarted = globalThis.GAME_HAS_STARTED;
        globalThis.EVENTS.fireEvent(new EV.ArmySpawnedEvent(_this.num, armyCatNum, pos, -1));
        return _this;
    }
    UnitArmy.prototype.addUnit = function (unitObj) {
        // set army in Unit model
        unitObj.army = this;
        // add to unit list
        this.units.push(unitObj);
        this.onUnitAdded(unitObj);
    };
    UnitArmy.prototype.addUnitOfType = function (typeId) {
        var newUnit = globalThis.UNITS.createUnit(typeId, this.pos);
        this.addUnit(newUnit);
    };
    Object.defineProperty(UnitArmy.prototype, "unitsTypeKeys", {
        /* ARMY GETTERS */
        get: function () {
            var unitKeys = [];
            for (var _i = 0, _a = this.units; _i < _a.length; _i++) {
                var unit = _a[_i];
                if (unit) {
                    unitKeys.push(unit.type.key);
                }
            }
            return unitKeys;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "armyTypeIcon", {
        get: function () { return this.armyCat.iconName; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "count", {
        get: function () { return this.units.length; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "length", {
        get: function () { return this.units.length; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "isCaravan", {
        get: function () { return this.armyCat.isCaravan; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "isInField", {
        get: function () {
            if (this.armyCat.isMilitia) {
                return false;
            }
            var isInCity = globalThis.CITIES && globalThis.CITIES.getCityAt(this.pos);
            if (isInCity) {
                return false;
            }
            return true;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "freeStorageSpace", {
        get: function () {
            var totalSpace = this.storageSpace;
            var takenSpace = this.storedResources.length;
            return totalSpace - takenSpace;
        },
        enumerable: false,
        configurable: true
    });
    /* return whether this army is of given ARMY_TYPE number */
    UnitArmy.prototype.isOfType = function (armyType) {
        return this.armyCat.num == armyType;
    };
    Object.defineProperty(UnitArmy.prototype, "unitTypes", {
        get: function () {
            var unitTypes = [];
            for (var _i = 0, _a = this.units; _i < _a.length; _i++) {
                var unit = _a[_i];
                if (!unitTypes.includes(unit.type)) {
                    unitTypes.push(unit.type);
                }
            }
            return unitTypes;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "ownerFaction", {
        get: function () {
            if (!globalThis.FACTIONS) {
                this.warn("cant get army owner faction: FactionManager not found");
                return;
            }
            var factionByKey = globalThis.FACTIONS.getFactionByKey(this._ownerKey);
            return factionByKey;
        },
        enumerable: false,
        configurable: true
    });
    UnitArmy.prototype.getRandomName = function () {
        // get prefix
        var randomPrefixes = ["Strong", "Zelous", "Loyal", "Free", "Bold", "Quick", "Victorious", "Vigilant", "Blessed"];
        // use city or empire name as prefix for militias, patrols or attack armies
        var prefix = RandomElement.of(randomPrefixes);
        // change prefix for player or city owned armies
        if (this.parentIsPlayer) {
            prefix = "Player";
        }
        if (this.parentCity) {
            prefix = this.parentCity.name;
        }
        // postfix
        var randomNouns = ["Army", "Force", "Expedition", "Soldiers", "Troops", "Banners", "Spears", "Swords"];
        var postfix = RandomElement.of(randomNouns);
        return prefix + " " + postfix;
    };
    /* ARMY STORAGE */
    UnitArmy.prototype.removeSingleStoredResource = function (resId) {
        var lastStored = null;
        var lastStoredIndex = -1;
        var _index = 0;
        for (var _i = 0, _a = this.storedResources; _i < _a.length; _i++) {
            var storedRes = _a[_i];
            if (storedRes.resId == resId) {
                lastStored = storedRes;
                lastStoredIndex = _index;
            }
            _index += 1;
        }
        if (lastStored) {
            this.storedResources.splice(lastStoredIndex, 1);
            this.log("1 ".concat(resId, " removed from storage"));
        }
    };
    /* STORAGE */
    UnitArmy.prototype.getStoredRes = function (resId, cheapest) {
        var ofResType = [];
        for (var _i = 0, _a = this.storedResources; _i < _a.length; _i++) {
            var stored = _a[_i];
            if (stored.resId == resId) {
                ofResType.push(stored);
            }
        }
        if (ofResType.length < 1) {
            this.error("cant get stored resource ".concat(resId, ": none in storage"));
            return;
        }
        if (!cheapest) {
            return ofResType[0];
        }
        else {
            var sortedByPrice = ofResType.sort(function (stored1, stored2) {
                if (stored1.price > stored2.price) {
                    return 1;
                }
                else if (stored1.price < stored2.price) {
                    return -1;
                }
                return 0;
            });
        }
    };
    Object.defineProperty(UnitArmy.prototype, "averageDamage", {
        /* AVERAGE UNIT PROPERTIES */
        get: function () {
            var totalDmg = 0;
            for (var _i = 0, _a = this.units; _i < _a.length; _i++) {
                var unit = _a[_i];
                var unitBaseDmg = unit.type.gear.weapon.baseAtk;
                totalDmg += unitBaseDmg;
            }
            // average unit damage
            return totalDmg / this.units.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "averageMovement", {
        get: function () {
            var totalMovement = 0;
            for (var _i = 0, _a = this.units; _i < _a.length; _i++) {
                var unit = _a[_i];
                var unitMovement = unit.type.movement;
                totalMovement += unitMovement;
            }
            return totalMovement / this.units.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "averageHealth", {
        get: function () {
            var totalHp = 0;
            for (var _i = 0, _a = this.units; _i < _a.length; _i++) {
                var unit = _a[_i];
                var unitHp = unit.hp;
                totalHp += unitHp;
            }
            return totalHp / this.units.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmy.prototype, "armyTypeName", {
        get: function () { return this.armyCat.name; },
        enumerable: false,
        configurable: true
    });
    /* ON EVENTS */
    UnitArmy.prototype.onUnitAdded = function (added) {
        if (this.parentCity) {
            // increase city might property
            this.parentCity.might += 1;
        }
    };
    /* EVEN HANDLING */
    UnitArmy.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
    };
    /* COMMANDS */
    UnitArmy.prototype.onTurn = function () {
    };
    UnitArmy.prototype.addCommand = function (cmd) {
    };
    UnitArmy.prototype.onCommand = function (cmd) {
    };
    /* MOVEMENT */
    UnitArmy.prototype.moveInDir = function (dirStr) {
    };
    /* STATIC & TOSTRING */
    UnitArmy.prototype.toString = function () {
        var unitsStr = this.unitsTypeKeys.join(", ");
        return "<Army \"".concat(this.name, "\" cat=").concat(this.armyCat.name, " units=[").concat(unitsStr, "]>");
    };
    Object.defineProperty(UnitArmy, "DEFAULT_MODEL", {
        get: function () { return "army_camp"; },
        enumerable: false,
        configurable: true
    });
    return UnitArmy;
}(OwnedLocation));
export { UnitArmy };
/* == ARMY CONSTANTS == */ 
//# sourceMappingURL=army.js.map