var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { AbilityList } from "../../../common/ability";
import { Loggable } from "../../../common/log";
import { Timespan } from "../../../common/timespan";
// unit entities
import { UnitGear } from "./gear/gear";
/* == UNIT TYPE MODEL == */
var UnitType = /** @class */ (function (_super) {
    __extends(UnitType, _super);
    // restricted cultures
    function UnitType(id, name, mountId, gearStr, isTrader, abilitiesStr, isWild, timespanStr, restrictedToCultures) {
        var _this = _super.call(this, "UnitType") || this;
        // flags
        _this.isTrader = false;
        _this.isWild = false;
        _this.restrictedToCultures = null;
        // available between years
        _this.timespan = null;
        _this._timespanStr = null;
        // abilities
        _this.abilities = null;
        // set basic properties
        _this.id = id;
        _this.name = name;
        // set unit mount, gear & abilities
        _this.setMountAndGear(mountId, gearStr);
        _this.abilities = AbilityList.FromString(abilitiesStr);
        // set optional flags
        if (isWild) {
            _this.isWild = true;
        }
        if (isTrader && isTrader == true) {
            _this.isTrader = true;
        }
        _this.timespan = Timespan.FromString(timespanStr);
        if (restrictedToCultures && restrictedToCultures.length > 0) {
            _this.restrictedToCultures = restrictedToCultures;
        }
        return _this;
    }
    /* UNIT TYPE INITIALIZING */
    UnitType.prototype.setMountAndGear = function (mountId, gearStr) {
        this.setMountById(mountId);
        this.setGearFromStr(gearStr);
    };
    UnitType.prototype.setGearFromStr = function (gearStr) {
        this._gearStr = gearStr;
        this.gear = new UnitGear(gearStr);
    };
    UnitType.prototype.setMountById = function (mountId) {
        this._mountId = mountId;
    };
    Object.defineProperty(UnitType.prototype, "weapon", {
        /* GEAR & ARMOR */
        get: function () {
            if (!globalThis.GEAR) {
                this.warn("cannot get weapon; UnitGearManager not found");
                return;
            }
            return globalThis.GEAR.getWeaponById(this.gear._weaponId);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitType.prototype, "armour", {
        get: function () {
            if (!globalThis.GEAR) {
                this.warn("cannot get armor; UnitGearManager not found");
                return;
            }
            return globalThis.GEAR.getArmorById(this.gear._armourId);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitType.prototype, "mount", {
        get: function () {
            if (!globalThis.GEAR) {
                this.warn("cannot get mount; UnitGearManager not found");
                return;
            }
            return globalThis.GEAR.getMountById(this._mountId);
        },
        enumerable: false,
        configurable: true
    });
    /* UNIT TYPE GETTERS */
    UnitType.prototype.toString = function () {
        return "<UnitType \"".concat(this.id, "\" isLand=").concat(this.isLand, " health=").concat(this.health, " mp=").concat(this.movement, " \n            mount=\"").concat(this._mountId, "\" gear=\"").concat(this._gearStr, "\" abilities=").concat(this.abilities.length, ">");
    };
    Object.defineProperty(UnitType.prototype, "isLand", {
        get: function () {
            if (this.mount) {
                return this.mount.isLand;
            }
            return true;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitType.prototype, "movement", {
        get: function () {
            if (this.mount) {
                var totalMovement = this.mount.baseMovement;
                // unit type has movement point bonus or malus?
                totalMovement += this.abilities.getValue(ABILITY_BONUS_MOVEMENT);
                totalMovement -= this.abilities.getValue(ABILITY_MINUS_MOVEMENT);
                return totalMovement;
            }
            return 2;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitType.prototype, "health", {
        get: function () {
            if (this.mount) {
                var totalHealth = this.mount.baseHealth;
                // unit type has health bonus or malus?
                totalHealth += this.abilities.getValue(ABILITY_BONUS_HEALTH);
                totalHealth -= this.abilities.getValue(ABILITY_MINUS_HEALTH);
                return totalHealth;
            }
            return 3;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitType.prototype, "cargoSpace", {
        get: function () {
            if (this.mount) {
                return this.mount.baseCargoSpace;
            }
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitType.prototype, "key", {
        get: function () { return this.id; },
        enumerable: false,
        configurable: true
    });
    return UnitType;
}(Loggable));
export { UnitType };
/* == UNIT TYPE CONSTANTS == */
export var ABILITY_BONUS_HEALTH = "bonus-health";
export var ABILITY_MINUS_HEALTH = "minus-health";
export var ABILITY_BONUS_MOVEMENT = "bonus-move";
export var ABILITY_MINUS_MOVEMENT = "minus-move";
//# sourceMappingURL=unit-type.js.map