var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// gui
import * as GUI_CONST from '../common/constants';
import { GuiRectangle } from '../common/rectangle';
import { ICON_WIDTH_TOTAL_PX, InlineIcon } from './icons';
// helper
import { ColorPalette } from '../../../helpers/colorpalette';
// babylon
import * as GUI from 'babylonjs-gui';
/* == DATAROW ELEMENT == */
/* a data row is a label including a key & value label, optionally preceded by an icon */
var SingleDataRow = /** @class */ (function (_super) {
    __extends(SingleDataRow, _super);
    function SingleDataRow(key, valueText, inlineIcon, overwriteElemId, clickable, valueColor, wideKeyLabel, plainKeyText) {
        if (plainKeyText === void 0) { plainKeyText = false; }
        var _this = 
        // get elemId and call super
        _super.call(this, "datarow-".concat(key)) || this;
        _this.wideKeyLabel = false;
        _this.plainKeyText = false; // show no ":"
        var elemId = "datarow-".concat(key);
        // set basic properties
        _this.name = elemId;
        _this.key = key;
        _this.valueText = valueText;
        // optional properties
        if (inlineIcon) {
            _this.inlineIconId = inlineIcon;
        }
        if (wideKeyLabel) {
            _this.wideKeyLabel = wideKeyLabel;
        }
        if (valueColor) {
            _this.valueColor = valueColor;
        }
        if (overwriteElemId) {
            elemId = overwriteElemId;
        }
        if (clickable) {
            _this.clickable = true;
        }
        if (plainKeyText) {
            _this.plainKeyText = true;
        }
        // set babylon control size
        _this.setSize(GUI_CONST.DEFAULT_INNER_WIDTH, GUI_CONST.DEFAULT_LABEL_HEIGHT);
        // initialize stack layout
        _this.addLayoutAndBg();
        _this.addIcon();
        _this.addKeyLabel();
        _this.addValueLabel();
        return _this;
        // this.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
    }
    /* add babylon Control directly */
    SingleDataRow.prototype._addControl = function (control) {
        control.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        _super.prototype.addControl.call(this, control);
    };
    /* add babylon control to inner container */
    SingleDataRow.prototype.addControl = function (control) {
        this.stackpanel.addControl(control);
        return this;
    };
    SingleDataRow.prototype.addLayoutAndBg = function () {
        // bg
        var bgImage = new GUI.Image("".concat(this.name, ".bg"), GUI_CONST.BG_PAPER);
        bgImage.width = this.width;
        // stackpanel
        this.stackpanel = new GUI.StackPanel("".concat(this.name, "-outter"));
        this.stackpanel.isVertical = false;
        this.stackpanel.width = this.width;
        this.stackpanel.paddingLeftInPixels = 16;
        this.stackpanel.paddingRightInPixels = 16;
        this._addControl(bgImage);
        this._addControl(this.stackpanel);
    };
    SingleDataRow.prototype.addIcon = function () {
        var iconElemId = "".concat(this.name, "-icon");
        var icon = new InlineIcon(iconElemId, this.inlineIconId);
        this.addControl(icon);
    };
    SingleDataRow.prototype.addKeyLabel = function () {
        var keyLabelId = "".concat(this.name, "-key");
        this.keyLabel = new DatarowKeyLabel(this, keyLabelId, this.key, null, this.clickable);
        this.addControl(this.keyLabel);
    };
    SingleDataRow.prototype.addValueLabel = function () {
        var valueLabelId = "".concat(this.name, "-value");
        this.valueLabel = new DatarowValueLabel(valueLabelId, this.valueText, this.key);
        this.addControl(this.valueLabel);
    };
    SingleDataRow.prototype.onElementClicked = function (elemId) {
        // globalThis.EVENTS.fireEvent( new EV.GuiElementClickedEvent(elemId) );
        console.log("<DataRow \"".concat(this.name, "\"> clicked element \"").concat(elemId, "\""));
        globalThis.GUI.onElementClicked(elemId);
    };
    return SingleDataRow;
}(GuiRectangle));
export { SingleDataRow };
/* == DATAROW KEY ELEMENT == */
var DatarowKeyLabel = /** @class */ (function (_super) {
    __extends(DatarowKeyLabel, _super);
    function DatarowKeyLabel(dataRow, elemId, key, customText, isClickable, plainKeyText) {
        var _this = _super.call(this, elemId, key + ": ") || this;
        // flags
        _this.isClickable = false;
        _this.plainKeyText = false;
        _this.dataRow = dataRow;
        if (customText) {
            _this.text = customText + ": ";
        }
        // sizing
        _this.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        _this.widthInPixels = LBL_KEY_WIDTH_PX;
        // and font style
        _this.setDefaultFontStyle();
        // clickable?
        if (isClickable) {
            _this.makeClickable();
        }
        // show now ":" after text
        if (plainKeyText) {
            _this.plainKeyText = true;
        }
        return _this;
    }
    DatarowKeyLabel.prototype.setDefaultFontStyle = function () {
        // set default color
        var defaultColorName = TEXT_COLOR_NAME_DEFAULT;
        this.color = ColorPalette.NameAsHexStr(defaultColorName);
        this.fontSize = GUI_CONST.TEXTSIZE_NORMAL;
        this.fontFamily = GUI_CONST.FONT_PARAGRAPH;
        this.textHorizontalAlignment = GUI.TextBlock.HORIZONTAL_ALIGNMENT_LEFT;
    };
    DatarowKeyLabel.prototype.makeClickable = function () {
        this.setClickableFontStyle();
        var _this_ = this;
        this.onPointerDownObservable.add(function () {
            var elemId = _this_.name;
            _this_.onElementClicked(elemId);
        });
    };
    DatarowKeyLabel.prototype.setClickableFontStyle = function () {
        this.isClickable = true;
        var linkColorName = TEXT_COLOR_NAME_LINK;
        this.color = ColorPalette.NameAsHexStr(linkColorName);
        this.fontWeight = "bold";
    };
    /* EVENT HANDLING */
    DatarowKeyLabel.prototype.onElementClicked = function (elemId) {
        this.dataRow.onElementClicked(elemId);
    };
    return DatarowKeyLabel;
}(GUI.TextBlock));
export { DatarowKeyLabel };
/* == DATAROW VALUE LABEL == */
var DatarowValueLabel = /** @class */ (function (_super) {
    __extends(DatarowValueLabel, _super);
    function DatarowValueLabel(elemId, valueStr, key) {
        var _this = _super.call(this, elemId, valueStr) || this;
        _this.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        _this.widthInPixels = LBL_VALUE_WIDTH_PX;
        _this.textHorizontalAlignment = GUI.TextBlock.HORIZONTAL_ALIGNMENT_LEFT;
        _this.setDefaultFontStyle();
        return _this;
    }
    DatarowValueLabel.prototype.setDefaultFontStyle = function () {
        this.fontSize = GUI_CONST.TEXTSIZE_NORMAL;
        this.fontFamily = GUI_CONST.FONT_HIGHLIGHTED;
    };
    return DatarowValueLabel;
}(GUI.TextBlock));
export { DatarowValueLabel };
/* == CONSTANTS == */
// row labels
export var LABELS_WIDTH_PX = GUI_CONST.DEFAULT_INNER_WIDTH_PX - ICON_WIDTH_TOTAL_PX;
export var LBL_KEY_WIDTH_PX = Math.round(LABELS_WIDTH_PX / 3);
export var LBL_VALUE_WIDTH_PX = LABELS_WIDTH_PX - LBL_KEY_WIDTH_PX;
// wide key label
export var LBL_KEY_WIDTH_WIDE_PX = Math.round(LABELS_WIDTH_PX / 2);
export var LBL_VALUE_WIDTH_NARROW_PX = LABELS_WIDTH_PX - LBL_KEY_WIDTH_WIDE_PX;
// row text colors
export var TEXT_COLOR_NAME_DEFAULT = "VALHALLA";
export var TEXT_COLOR_NAME_LINK = "ROYALBLUE";
//# sourceMappingURL=data-row.js.map