var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { RandomDice, RandomElement, RandomInt, RandomWorldPos } from "../../helpers/random";
import * as EV from "../game-events";
// entities
import { UnitArmy } from "../entities/unit/army/army";
import { ArmyCategory } from "../entities/unit/army/army-category";
// manager
import { AbstractManager } from "./abstract-manager";
var ArmyManager = /** @class */ (function (_super) {
    __extends(ArmyManager, _super);
    function ArmyManager() {
        var _this = _super.call(this, "Army") || this;
        _this.armies = [];
        _this._armyTypes = [];
        // flags
        _this.hasPlayerArmyCreated = false;
        _this.hasMilitiasCreated = false;
        _this.hasEmpireArmiesCreated = false;
        return _this;
    }
    /* ARMY CREATION */
    ArmyManager.prototype.createNewArmy = function (armyType, pos, armyOwnerKey) {
        var army = new UnitArmy(armyType, pos);
        army.num = this.armies.length;
        if (armyOwnerKey) {
            army._ownerKey = armyOwnerKey;
        }
        return this._addArmy(army);
    };
    ArmyManager.prototype._addArmy = function (armyObj) {
        this.armies.push(armyObj);
        this._armyTypes.push(armyObj.armyCat.num);
        return armyObj;
    };
    /* ARMY GETTERS */
    ArmyManager.prototype.getArmyByNum = function (armyNum) {
        var armyWithNum = this.armies[armyNum];
        if (!armyWithNum) {
            this.warn("cant find army with num ".concat(armyNum));
        }
        return armyWithNum;
    };
    ArmyManager.prototype.getArmiesAtPos = function (worldPos) {
        var armiesAtPos = [];
        for (var _i = 0, _a = this.armies; _i < _a.length; _i++) {
            var army = _a[_i];
            if (army.pos.equals(worldPos)) {
                armiesAtPos.push(army);
            }
        }
        return armiesAtPos;
    };
    ArmyManager.prototype.getArmiesByFactionKey = function (factionKey) {
        var factionArmies = [];
        for (var _i = 0, _a = this.armies; _i < _a.length; _i++) {
            var army = _a[_i];
            if (army._ownerKey == factionKey) {
                factionArmies.push(army);
            }
        }
        return factionArmies;
    };
    ArmyManager.prototype.getArmiesByFaction = function (faction) {
        return this.getArmiesByFactionKey(faction.key);
    };
    ArmyManager.prototype.getOutsideArmies = function () {
        var outsideArmies = [];
        for (var _i = 0, _a = this.armies; _i < _a.length; _i++) {
            var army = _a[_i];
            if (army.isInField) {
                outsideArmies.push(army);
            }
        }
        return outsideArmies;
    };
    ArmyManager.FilterByArmyCatNum = function (armies, armyCatNum) {
        var armiesOfCat = [];
        for (var _i = 0, armies_1 = armies; _i < armies_1.length; _i++) {
            var army = armies_1[_i];
            if (army.armyCat.num == armyCatNum) {
                armiesOfCat.push(army);
            }
        }
        return armiesOfCat;
    };
    ArmyManager.prototype.filterByArmyCatNum = function (armies, armyCatNum) {
        return ArmyManager.FilterByArmyCatNum(armies, armyCatNum);
    };
    /* shortforms */
    ArmyManager.prototype.ByNum = function (num) { return this.getArmyByNum(num); };
    ArmyManager.prototype.AtPos = function (pos) { return this.getArmiesAtPos(pos); };
    /* CREATE NEW ARMIES */
    /* create a new army in a city (can be militia or patrol) */
    ArmyManager.prototype.createNewCityArmy = function (city, isPatrol) {
        var ownerFactionKey = city._factionKey;
        var armyCatNum = ArmyCategory.CITY_MILITIA;
        if (isPatrol && isPatrol == true) {
            armyCatNum = ArmyCategory.CITY_PATROL;
        }
        var army = this.createNewArmy(armyCatNum, city.pos, ownerFactionKey);
        army.parentCity = city;
        army.home = city;
        if (!isPatrol) {
            city.militia = army;
        }
        return army;
    };
    ArmyManager.prototype.createPlayerArmy = function () {
        this.log("creating player starting army");
        // abort when player has already an army created
        if (this.hasPlayerArmyCreated) {
            return;
        }
        // create new UnitArmy object
        var playerFamilyKey = globalThis.PLAYER.family.key;
        this._playerArmy = this.createNewArmy(ArmyCategory.CARAVAN_PLAYER, globalThis.PLAYER.pos, playerFamilyKey);
        // add to player
        globalThis.PLAYER.setArmy(this._playerArmy);
        this._playerArmy.parentIsPlayer = true;
        // add donkey starting unit
        this._playerArmy.addUnitOfType("DN");
        // flag player army creation as done
        this.hasPlayerArmyCreated = true;
        return this._playerArmy;
    };
    ArmyManager.prototype.createNewFactionArmy = function (faction, startPos, isPatrol) {
        var armyCatNum = ArmyCategory.FACTION_ARMY;
        if (isPatrol) {
            armyCatNum = ArmyCategory.FACTION_PATROL;
        }
        var factionKey = faction.key;
        var factionArmy = this.createNewArmy(armyCatNum, startPos, factionKey);
        // add to faction
        factionArmy.parentFaction = faction;
        faction.factionArmies.push(factionArmy);
        return factionArmy;
    };
    /* STARTING ARMY CREATION */
    ArmyManager.prototype.createStartingArmies = function () {
        // already created all starting armies? abort
        if (this.hasAllArmiesCreated) {
            return;
        }
        this.log("creating starting armies..");
        // create city miltia armies
        if (!this.hasMilitiasCreated) {
            this._createCityMilitias();
        }
        // create empire armies
        if (!this.hasEmpireArmiesCreated) {
            this._createEmpireArmies();
        }
    };
    ArmyManager.prototype.addRndMilitaryUnitTo = function (army) {
        var unitTypeId = this.getRandomMilityUnitTypeId();
        army.addUnitOfType(unitTypeId);
    };
    ArmyManager.prototype.getRandomMilityUnitTypeId = function () {
        var militaryTypes = globalThis.UNIT_TYPES.getMilitaryTypes();
        return RandomElement.of(militaryTypes).id;
    };
    ArmyManager.prototype.createRndCityMilitia = function (city) {
        if (this.hasMilitiasCreated) {
            return;
        }
        var army = this.createNewCityArmy(city, false);
        var startingUnitCount = RandomElement.of([1, 2, 3]);
        for (var unitNum = 0; unitNum < startingUnitCount; unitNum++) {
            this.addRndMilitaryUnitTo(army);
        }
        return army;
    };
    ArmyManager.prototype._createCityMilitias = function () {
        var citiesLoaded = globalThis.CITIES && globalThis.CITIES.getAll().length > 0;
        var militiaArmies = [];
        if (!citiesLoaded) {
            this.warn("cant create city militias; CityManager not initialized");
            return;
        }
        var citiesCount = globalThis.CITIES.getAll().length;
        var firstMilitiaArmyNum = this.armies.length;
        for (var _i = 0, _a = globalThis.CITIES.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            var militiaArmy = this.createRndCityMilitia(city);
            militiaArmies.push(militiaArmy);
        }
        var militiaArmiesCount = this.armies.length - firstMilitiaArmyNum;
        this.log("".concat(militiaArmiesCount, " for ").concat(citiesCount, " cities created"));
        this.hasMilitiasCreated = true;
        return militiaArmies;
    };
    ArmyManager.prototype._createEmpireArmies = function () {
        // check for FactionManager
        if (!globalThis.FACTIONS) {
            this.warn("cant create empire armies: FactionManager not found");
            return false;
        }
        var firstEmpireArmyNum = this.armies.length;
        for (var _i = 0, _a = globalThis.FACTIONS.empires; _i < _a.length; _i++) {
            var empire = _a[_i];
            this._createEmpireStartingArmies(empire);
        }
        var empireArmiesCount = this.armies.length - firstEmpireArmyNum;
        this.log("created ".concat(empireArmiesCount, " empire armies"));
        this.hasEmpireArmiesCreated = true;
        return true;
    };
    ArmyManager.prototype._createEmpireStartingArmies = function (empire) {
        var startingArmiesCount = RandomInt.below(3);
        var startingArmies = [];
        for (var empireArmyNum = 0; empireArmyNum <= startingArmiesCount; empireArmyNum++) {
            var isPatrol = RandomDice.roll(2);
            var empireArmy = this._createSingleEmpireArmy(empire, isPatrol);
            startingArmies.push(empireArmy);
        }
        return startingArmies;
    };
    ArmyManager.prototype._createSingleEmpireArmy = function (empire, isPatrol) {
        var army = this.createNewFactionArmy(empire, null, isPatrol);
        // empire has capital? set as home
        var empireCapital = empire.capitalCity;
        if (!empireCapital) {
            this.warn("creating empire \"".concat(empire.key, "\" army: no home assignable"));
        }
        else {
            army.home = empireCapital;
            // set rnd start pos
            var rndStartPos = RandomWorldPos.around(empireCapital.pos, 4);
            army.pos = rndStartPos;
        }
        // add 1 or 2 random military units
        var startingUnitsCount = RandomInt.below(3);
        for (var empireUnitNum = 0; empireUnitNum < startingUnitsCount; empireUnitNum++) {
            this.addRndMilitaryUnitTo(army);
        }
        return army;
    };
    Object.defineProperty(ArmyManager.prototype, "hasAllArmiesCreated", {
        /* ARMY MANAGER GETTERS */
        get: function () {
            var allArmiesCreated = this.hasPlayerArmyCreated && this.hasMilitiasCreated && this.hasEmpireArmiesCreated;
            if (allArmiesCreated) {
                this.onAllArmiesCreated();
                return true;
            }
            return false;
        },
        enumerable: false,
        configurable: true
    });
    /* INTERNAL EVENTS */
    ArmyManager.prototype.onCityListCreated = function () {
        this.createPlayerArmy();
    };
    ArmyManager.prototype.onEmpireCapitalsAssigned = function () {
        this.createStartingArmies();
    };
    ArmyManager.prototype.onAllArmiesCreated = function () {
        this.log("finished creation of ".concat(this.armies.length, " armies for player, cities & faction"));
    };
    /* EVENT HANDLING */
    ArmyManager.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        if (ev.id == EV.ID_UNIT_TYPES_INITIALIZED) {
            this.createStartingArmies();
        }
        else if (ev.id == EV.ID_PLAYER_MOVED) {
            this.createStartingArmies();
        }
    };
    Object.defineProperty(ArmyManager.prototype, "hasLoaded", {
        get: function () { return this.armies.length > 1; },
        enumerable: false,
        configurable: true
    });
    return ArmyManager;
}(AbstractManager));
export { ArmyManager };
//# sourceMappingURL=army-manager.js.map