var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// events
import { BaseEventListener } from '../../common/events';
import * as EV from "../game-events";
// babylon
import * as BABYLON from 'babylonjs';
import 'babylonjs-loaders';
import { Asset } from '../../common/asset';
/* ASSET MANAGER */
var AssetsLoader = /** @class */ (function (_super) {
    __extends(AssetsLoader, _super);
    function AssetsLoader() {
        var _this = _super.call(this, "AssetManager") || this;
        _this.assetNames = [];
        _this.assets = [];
        _this.assetsToLoad = 0;
        _this.assetsLoaded = 0;
        _this.log("AssetsLoader created");
        return _this;
    }
    AssetsLoader.prototype.loadAssets = function () {
        var assetsCount = this.assets.length;
        this.log("initializing loading of ".concat(assetsCount, " assets .."));
        this.createEmptyAssets();
        // start model loading
        var loadingAssetCount = 0;
        for (var _i = 0, _a = this.assets; _i < _a.length; _i++) {
            var asset = _a[_i];
            var loadingStarted = this.loadSingleAsset(asset);
            if (loadingStarted) {
                loadingAssetCount += 1;
            }
        }
        this.log("loading of ".concat(loadingAssetCount, " assets started"));
    };
    AssetsLoader.prototype.createEmptyAssets = function () {
        for (var _i = 0, MODELS_TO_IMPORT_1 = MODELS_TO_IMPORT; _i < MODELS_TO_IMPORT_1.length; _i++) {
            var modelFileName = MODELS_TO_IMPORT_1[_i];
            var empty = new Asset(modelFileName);
            // add empty asset
            this.assets.push(empty);
            this.assetNames.push(empty.name);
            // this.startModelLoading(modelFileName);
            this.assetsToLoad += 1;
        }
    };
    AssetsLoader.prototype.loadSingleAsset = function (asset) {
        var modelFileName = asset.fileName;
        // this.log(`start loading of ${modelFileName} ..`);
        if (!globalThis.SCENE) {
            this.error("cant load ".concat(modelFileName, "; scene not yet initialized"));
            return false;
        }
        var rootDir = MODEL_DIRECTORY;
        var assetName = modelFileName.split(".")[0];
        BABYLON.SceneLoader.ImportMesh(assetName, rootDir, modelFileName, globalThis.SCENE, function (newMeshes) {
            // console.log(`LOADED MESH ${assetName} meshes "${newMeshes.length}`);
            globalThis.ASSETS.onSingleAssetLoaded(assetName, newMeshes);
        }),
            function (error) {
                console.error(error);
            };
        return true;
    };
    AssetsLoader.prototype.onSingleAssetLoaded = function (assetName, meshes) {
        this.log("model ".concat(assetName, " loaded: ").concat(meshes.length, " meshes"));
        var loadedAsset = this.getAsset(assetName);
        var assetsMgr = this;
        meshes[0].name = "asset-" + assetName;
        if (meshes.length > 1) {
            meshes[1].id = assetName + "-1";
        }
        loadedAsset.setMeshes(meshes);
        // renew loading counters
        // this.assetsLoaded += 1;
        this.assetsToLoad -= 1;
        if (this.allAssetsLoaded) {
            assetsMgr.onAllAssetsLoaded();
        }
    };
    AssetsLoader.prototype.onAllAssetsLoaded = function () {
        this.log("all assets loaded!");
        // before world placement
        // globalThis.FACTIONS.onBeforeWorldPlacement();
        // inform managers
        globalThis.WORLD.onAllAssetsLoaded();
        // fire assets loaded event
        globalThis.EVENTS.fireEvent(new EV.AssetsLoadedEvent(this.assetsLoaded));
    };
    Object.defineProperty(AssetsLoader.prototype, "allAssetsLoaded", {
        get: function () {
            return this.assetsToLoad == 0;
        },
        enumerable: false,
        configurable: true
    });
    AssetsLoader.prototype.getAsset = function (assetName, autoscale) {
        if (autoscale != true) {
            autoscale = false;
        }
        var index = this.assetNames.indexOf(assetName);
        var asset = this.assets[index];
        if (!asset) {
            this.error("cant get asset ".concat(assetName, ": is unknown"));
            return;
        }
        else if (asset.errorMessage && asset.errorMessage.length > 0) {
            this.error("cant get asset ".concat(assetName, ": loading error: ").concat(asset.errorMessage));
            return;
        }
        return asset;
    };
    return AssetsLoader;
}(BaseEventListener));
export { AssetsLoader };
/* == MODEL NAME CONSTANTS == */
export var CURSOR_MODELS = ["cursor_highlighted.obj", "cursor_move_green.obj", "cursor_move_red.obj"];
export var CITY_MODELS = [
    "rural_townhall.obj", "city_asian.obj", "city_tribal.obj", "city_venetian.obj", "city_hanseatic.obj", "city_hellenic.obj",
    "city_mideast.obj", "city_slavic.obj", "city_weur.obj", "city_elven.obj", "city_colonial.obj",
    "city_dwarven.obj", "city_barbarian.obj", "city_cloister.obj", "city_roman.obj", "city_evil.obj",
    // unique citiy models
    "city_vivec.obj"
];
export var ARMY_MODELS = [
    "rider.obj", "ship_smallest.obj", "army_camp.obj", "army_fort.obj"
];
export var FEATURE_MODELS = [
    "forest.obj", "treasure.obj", "volcano.obj", "temple.obj", "tipis.obj", "shipwreck.obj",
    "village.obj", "oasis.obj", "arctic_peak.obj", "shrooms.obj", "ruins.obj", "watchtower.obj",
    "watchtower_white.obj", "farmstead.obj", "ice_floe.obj", "lighttower.obj", "pyramids.obj", "island.obj" // features 3
];
export var MODELS_TO_IMPORT = CURSOR_MODELS.concat(FEATURE_MODELS, CITY_MODELS, ARMY_MODELS);
/* == ASSET LOADER CONSTANTS == */
export var MODEL_DIRECTORY = "./res/models/";
//# sourceMappingURL=asset-loader.js.map