var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { RandomElement } from "../../helpers/random";
import * as EV from "../game-events";
import { Character } from "../entities/character/character";
// manager
import { AbstractManager } from "./abstract-manager";
var CharacterManager = /** @class */ (function (_super) {
    __extends(CharacterManager, _super);
    function CharacterManager() {
        var _this = _super.call(this, "CharManager") || this;
        _this.nameList = null;
        _this.characters = [];
        _this.hasCreatedLeaders = false;
        return _this;
    }
    /* INITIALIZATION */
    CharacterManager.prototype.onLanguagesLoaded = function (nameListObj) {
        this.log("Character name list initialized");
        this.nameList = nameListObj;
        // inform faction manager
        globalThis.FACTIONS.onNameListLoaded();
        // create leaders
        if (!this.hasCreatedLeaders) {
            this.createCityLeaders();
        }
    };
    Object.defineProperty(CharacterManager.prototype, "hasNameListLoaded", {
        get: function () {
            if (this.nameList) {
                var languagesCount = Object.keys(this.nameList).length;
                if (languagesCount > 1) {
                    return true;
                }
            }
            return false;
        },
        enumerable: false,
        configurable: true
    });
    CharacterManager.prototype.createPlayerCharacter = function () {
    };
    CharacterManager.prototype.createCityLeaders = function () {
        if (this.hasCreatedLeaders) {
            return;
        }
        if (!this.hasNameListLoaded) {
            this.warn("cant create city leaders; name list not loaded yet");
            return;
        }
        var citiesCount = globalThis.CITIES.citiesCount;
        var createdCount = 0;
        this.log("creating leaders for ".concat(citiesCount, " cities"));
        for (var _i = 0, _a = globalThis.CITIES.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            this.createCharacterIn(city, true);
            createdCount += 1;
        }
        this.log("created ".concat(createdCount, " character leaders in cities"));
        this.hasCreatedLeaders = true;
    };
    /* CHARACTER GETTERS */
    CharacterManager.prototype.getByNum = function (charNum) {
        var characterObj = this.characters[charNum];
        return characterObj;
    };
    CharacterManager.prototype.getRandom = function () {
        return RandomElement.of(this.characters);
    };
    CharacterManager.prototype.getRandomName = function (language, isFemale) {
        if (!this.hasNameListLoaded) {
            this.warn("cant generaate random name; name list not found");
            return;
        }
        var languageNameList = this.nameList[language];
        // family name
        var rndFamilyName = "";
        if (languageNameList[2].length > 0) {
            rndFamilyName = RandomElement.of(languageNameList[2]);
        }
        // own name
        var rndCharName = RandomElement.of(languageNameList[0]);
        if (isFemale) {
            rndCharName = RandomElement.of(languageNameList[1]);
        }
        return rndCharName + " " + rndFamilyName;
    };
    CharacterManager.prototype.getCityLeaders = function () {
        var citiesAreLoaded = globalThis.CITIES && globalThis.CITIES.citiesCount > 0;
        if (!citiesAreLoaded) {
            this.warn("cant get city leaders: CityManager not initialized");
            return;
        }
        var cityLeaders = [];
        for (var _i = 0, _a = globalThis.CITIES.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            var cityLeader = city.leader;
            if (cityLeader) {
                cityLeaders.push(cityLeader);
            }
        }
        return cityLeaders;
    };
    /* NAME LIST */
    CharacterManager.prototype.getRandomFamilyName = function (langId) {
        if (!this.hasNameListLoaded) {
            this.error("NAME LIST NOT LOADED");
            return;
        }
        var familyNames = this.nameList[langId][2];
        return RandomElement.of(familyNames);
    };
    /* CHARACTER CREATION */
    CharacterManager.prototype.addCharacter = function (characterObj) {
        this.characters.push(characterObj);
        return characterObj;
    };
    CharacterManager.prototype.createCharacterIn = function (city, isLeader) {
        var charNum = this.characters.length;
        var isFemale = city.num % 2 == 0;
        var cityLanguage = city.language;
        // city is owned by a faction?
        var cityOwnerNum = -1;
        if (city.owner && city.owner.num >= 0) {
            cityOwnerNum = city.owner.num;
        }
        // create new Character object
        var rndChar = new Character(charNum, cityOwnerNum, cityLanguage, null, isFemale);
        // assign home & pos
        rndChar.home = city;
        rndChar.pos = city.pos;
        // add to city characters and make leader when required
        if (isLeader) {
            city.leader = rndChar;
        }
        city.localCharacters.push(rndChar);
        // add to character list
        this.addCharacter(rndChar);
        return rndChar;
    };
    /* EVENT HANDLING */
    CharacterManager.prototype.onEvent = function (ev) {
        var success = _super.prototype.onEvent.call(this, ev);
        // cities loaded?
        if (ev.id == EV.ID_CITY_LIST_LOADED) {
            this.log("CITY LIST LOADED");
            this.createCityLeaders();
        }
        // language file loaded?
        if (ev.id == EV.ID_FILE_JSON_LOADED) {
            if (ev.properties.getStr("shortPath") == "default-languages") {
                var languageJson = globalThis.FILES.jsonContent["default-languages"];
                this.onLanguagesLoaded(languageJson);
            }
        }
        return true;
    };
    return CharacterManager;
}(AbstractManager));
export { CharacterManager };
//# sourceMappingURL=character-manager.js.map