var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { RandomElement } from "../../helpers/random";
import * as EV from "../game-events";
// models
import { City } from "../entities/city/city";
// manager
import { AbstractManager } from "./abstract-manager";
var CityManager = /** @class */ (function (_super) {
    __extends(CityManager, _super);
    function CityManager(generate) {
        var _this = _super.call(this, "City") || this;
        _this._names = [];
        _this.cities = [];
        _this._placedCities = [];
        _this.bldTypes = [];
        return _this;
    }
    /* GET & SET CITIES */
    CityManager.prototype.addCity = function (city) {
        if (!city.num) {
            city.num = this.cities.length;
        }
        this._names.push(city.name);
        this.cities.push(city);
    };
    CityManager.prototype.createCity = function (name, pos, culture, primaryResource) {
        var newCity = new City(null, name, pos, culture, primaryResource);
        this.addCity(newCity);
    };
    CityManager.prototype.getAll = function () {
        return this.cities;
    };
    Object.defineProperty(CityManager.prototype, "citiesCount", {
        get: function () {
            if (this.cities && this.cities.length > 0) {
                return this.cities.length;
            }
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    /* GET BY PROPERTY */
    CityManager.prototype.getByNum = function (num) {
        var city = this.cities[num];
        return city;
    };
    CityManager.prototype.getByKey = function (key) {
        for (var _i = 0, _a = this.cities; _i < _a.length; _i++) {
            var city = _a[_i];
            if (city.key == key) {
                return city;
            }
        }
        return null;
    };
    CityManager.prototype.getByName = function (name) {
        var cityIndex = this._names.indexOf(name);
        return this.cities[cityIndex];
    };
    CityManager.prototype.getCityAt = function (worldPos) {
        for (var _i = 0, _a = this.cities; _i < _a.length; _i++) {
            var city = _a[_i];
            if (city.pos.equals(worldPos)) {
                return city;
            }
        }
        return null;
    };
    CityManager.prototype.getCitiesByFactionKey = function (factionKey) {
        var factionCities = [];
        for (var _i = 0, _a = this.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            if (city._factionKey && city._factionKey == factionKey) {
                factionCities.push(city);
            }
        }
        return factionCities;
    };
    // shortforms
    CityManager.prototype.ByNum = function (num) { return this.getByNum(num); };
    CityManager.prototype.ByKey = function (key) { return this.getByKey(key); };
    CityManager.prototype.ByName = function (name) { return this.getByName(name); };
    CityManager.prototype.AtPos = function (pos) { return this.getCityAt(pos); };
    CityManager.prototype.OfOwner = function (factionKey) { return this.getCitiesByFactionKey(factionKey); };
    /* CITY MESH */
    CityManager.prototype.getMeshes = function () {
        var cityMeshes = [];
        for (var _i = 0, _a = this.cities; _i < _a.length; _i++) {
            var city = _a[_i];
            if (city.mesh) {
                cityMeshes.push(city.mesh);
            }
        }
        return cityMeshes;
    };
    CityManager.prototype.getRandom = function () {
        return RandomElement.of(this.cities);
    };
    /* public load(): number {
        let cityAmount: number = 0;
        return cityAmount;
    } */
    CityManager.prototype._onTurnFinished = function () {
        this.log("tick");
        for (var _i = 0, _a = this.cities; _i < _a.length; _i++) {
            var city = _a[_i];
            city.onTurn();
        }
    };
    Object.defineProperty(CityManager, "RANDOM_AMOUNT", {
        get: function () {
            return 20;
        },
        enumerable: false,
        configurable: true
    });
    /* notifies a tile that a city is standing on it */
    CityManager.prototype.linkCityTiles = function () {
        for (var _i = 0, _a = this.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            var tileAt = globalThis.MAP.tileAtPos(city.pos);
            tileAt.city = city;
        }
        this.log("flagged city tiles");
    };
    CityManager.prototype.makeCityRegions = function () {
        // link neighboring tiles
        this.log('TODO generate city regions');
    };
    CityManager.prototype.initializeCityEmpire = function () {
        var ownedCitiesAmount = 0;
        for (var _i = 0, _a = this.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            if (city._factionKey) { // when city has an owner..
                var empireObj = globalThis.FACTIONS.getEmpireByKey(city._factionKey);
                if (empireObj) {
                    // .. that is an empire
                    empireObj.giveCityOwnership(city, true);
                    ownedCitiesAmount += 1;
                }
            }
        }
        this.log("given empire ownership to ".concat(ownedCitiesAmount, " cities"));
    };
    /* CITY FEATURES */
    CityManager.prototype.initializeCityFeatures = function () {
        for (var _i = 0, _a = this.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            city.loadFeatures();
        }
    };
    /* BUILDINGS */
    CityManager.prototype.getAllBuildingTypes = function () {
        return globalThis.CONSTRUCTABLE.getAll();
    };
    CityManager.prototype.getBuildingTypesByCat = function (bldCatNum) {
        var catBuildings = [];
        for (var _i = 0, _a = this.getAllBuildingTypes(); _i < _a.length; _i++) {
            var bldType = _a[_i];
            if (bldCatNum == bldType.category.num) {
                catBuildings.push(bldType);
            }
        }
        return catBuildings;
    };
    /* ON EVENTS */
    CityManager.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        // when cities are placed -> link to tile
        if (ev.id == EV.ID_CITIES_PLACED) {
            this.linkCityTiles();
        }
    };
    /* fired by world scene after all cities have been added */
    CityManager.prototype.onCitiesLoaded = function () {
        // load city features
        this.initializeCityFeatures();
        // initialize cities ownership
        this.initializeCityEmpire();
        globalThis.ARMIES.onCityListCreated();
        globalThis.FACTIONS.onCitiesLoaded();
        // initialize city regions
        this.makeCityRegions();
        // initialize city connections
        globalThis.CONN.makeCityConnections();
        // fire event after loading features and owner faction
        // globalThis.EVENTS.fireEvent( new EV.CityListLoadedEvent( citiesCount ), true);
        // inform trafe & unit manager
        globalThis.TRADE.onCitiesLoaded();
        globalThis.UNITS.onCitiesLoaded();
        // globalThis.CHARS.createCityLeaders();
    };
    return CityManager;
}(AbstractManager));
export { CityManager };
//# sourceMappingURL=city-manager.js.map