var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { RandomInt, RandomDice } from "../../helpers/random";
import { Empire } from "../entities/faction/empire";
import { Family } from "../entities/faction/family";
import { Cultures } from "../entities/faction/culture";
// data
import { CustomFamiliesLoader, WorldEmpireLoader } from "./world-loaders";
// manager
import { AbstractManager } from "./abstract-manager";
import { ArmyManager } from "./army-manager";
/* == FACTION MANAGER == */
var FactionManager = /** @class */ (function (_super) {
    __extends(FactionManager, _super);
    function FactionManager() {
        var _this = _super.call(this, "Faction") || this;
        // families
        _this.families = [];
        _this.playerFamily = null;
        // empires
        _this.empires = [];
        _this._empireKeys = [];
        _this.cultures = null;
        // flags
        _this.hasFamilyHomesLoaded = false;
        _this.hasStartingArmiesCreated = false;
        _this.hasEmpireCapitalsAssigned = false;
        // private
        _this.lastFactionNum = -1;
        _this._empireCapitalKeys = {};
        _this._empireCapitalsAssignedCount = 0;
        _this.loadCultures();
        _this.createDefaultEmpires();
        _this.loadEmpiresFromTsFile();
        _this.loadOrGenerateFamilies();
        return _this;
    }
    FactionManager.prototype.loadOrGenerateFamilies = function () {
        var customFamilies = new CustomFamiliesLoader().load();
        if (customFamilies && customFamilies.length > 0) {
            var isFirstFamily = true;
            for (var _i = 0, customFamilies_1 = customFamilies; _i < customFamilies_1.length; _i++) {
                var family = customFamilies_1[_i];
                if (isFirstFamily) {
                    family.isAi = false;
                }
                this.addFamily(family);
                isFirstFamily = false;
            }
        }
        else {
            this.createRandomFamilies();
        }
        return null;
    };
    /* FACTION GETTERS */
    FactionManager.prototype.getFactionByNum = function (factionNum) {
        if (factionNum > this.lastFactionNum) {
            this.error("cant get faction with num ".concat(factionNum, "; only ").concat(this.lastFactionNum, " factions loaded"));
            return;
        }
        for (var _i = 0, _a = this.allFactions; _i < _a.length; _i++) {
            var faction = _a[_i];
            if (faction.num == factionNum) {
                return faction;
            }
        }
        return null;
    };
    FactionManager.prototype.getEmpireByKey = function (empireKey) {
        for (var _i = 0, _a = this.empires; _i < _a.length; _i++) {
            var empire = _a[_i];
            if (empire.key == empireKey) {
                return empire;
            }
        }
        return null;
    };
    Object.defineProperty(FactionManager.prototype, "allFactions", {
        get: function () {
            var allFactions = [];
            // collect families
            for (var _i = 0, _a = this.families; _i < _a.length; _i++) {
                var family = _a[_i];
                allFactions.push(family);
            }
            // .. & empires
            for (var _b = 0, _c = this.empires; _b < _c.length; _b++) {
                var empire = _c[_b];
                allFactions.push(empire);
            }
            // and return all as IFaction array
            return allFactions;
        },
        enumerable: false,
        configurable: true
    });
    FactionManager.prototype.getFactionByKey = function (factionKey) {
        // family?
        if (factionKey.startsWith("F") && factionKey.length == 2) {
            var familyNum = parseInt(factionKey.replace("F", ""));
            return this.getFamilyByNum(familyNum);
        }
        // empire?
        return this.getEmpireByKey(factionKey);
    };
    /* shortforms */
    FactionManager.prototype.ByKey = function (key) { return this.getFactionByKey(key); };
    /* FAMILY MANAGEMENT */
    FactionManager.prototype.addFamily = function (family) {
        var familyNum = this.families.length;
        family.setNum(familyNum);
        this.families.push(family);
        if (!family.isAi) {
            this.playerFamily = family;
        }
    };
    FactionManager.prototype.addRandomFamily = function (name, randomHome) {
        var playersAmount = this.families.length;
        var isPlayer = playersAmount == 0;
        var rndFamily = Family.random();
        if (randomHome) {
            rndFamily.hasRandomHome = true;
        }
        if (isPlayer) {
            rndFamily.isAi = false;
            this.playerFamily = rndFamily;
        }
        this.addFamily(rndFamily);
        return rndFamily;
    };
    FactionManager.prototype.createRandomFamilies = function () {
        var families = [];
        for (var familyNum = 0; familyNum < RND_FAMILY_COUNT; familyNum++) {
            this.addRandomFamily(null, true);
        }
        return families;
    };
    FactionManager.prototype.getPlayerFamily = function () {
        if (this.families.length == 0) {
            this.error("cant get player faction; families not generated");
            return;
        }
        return this.families[0];
    };
    FactionManager.prototype.getFamilyByNum = function (num) {
        var foundFamily = this.families[num];
        if (!foundFamily) {
            this.error("no family ".concat(num, " found in ").concat(this.getFamilyNums()));
            return;
        }
        return foundFamily;
    };
    FactionManager.prototype.getFamilyNums = function () {
        var nums = [];
        for (var _i = 0, _a = this.families; _i < _a.length; _i++) {
            var family = _a[_i];
            nums.push(family.num);
        }
        return nums;
    };
    FactionManager.prototype.getFactionArmies = function (faction) {
        if (!globalThis.ARMIES) {
            this.warn("cant get armies of ".concat(faction.name, ": ArmyManager not found"));
            return;
        }
        return globalThis.ARMIES.getArmiesByFaction(faction);
    };
    FactionManager.prototype.getFactionArmiesByCat = function (faction, armyCatNum) {
        var factionArmies = this.getFactionArmies(faction);
        var factionArmiesOfCat = ArmyManager.FilterByArmyCatNum(factionArmies, armyCatNum);
        return factionArmiesOfCat;
    };
    /* EMPIRE MANAGEMENT */
    FactionManager.prototype.addEmpire = function (empire) {
        this._empireKeys.push(empire.key);
        this.empires.push(empire);
    };
    FactionManager.prototype.getEmpireCities = function (empireKey) {
        var empireCities = [];
        for (var _i = 0, _a = globalThis.CITIES.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            if (city._factionKey && city._factionKey == empireKey) {
                empireCities.push(city);
            }
        }
        return empireCities;
    };
    FactionManager.prototype.getEmpires = function (hideDefault) {
        var empires = [];
        for (var _i = 0, _a = this.empires; _i < _a.length; _i++) {
            var empire = _a[_i];
            var hideEmpire = false;
            if (hideDefault == true && empire.isDefault) {
                hideEmpire = true;
            }
            if (!hideEmpire) {
                empires.push(empire);
            }
        }
        return empires;
    };
    FactionManager.prototype.getEmpireKeys = function (hideDefault) {
        var empireKeys = [];
        for (var _i = 0, _a = this.getEmpires(hideDefault); _i < _a.length; _i++) {
            var nonHiddenEmpire = _a[_i];
            empireKeys.push(nonHiddenEmpire.key);
        }
        return empireKeys;
    };
    /* LOADING */
    FactionManager.prototype.loadCultures = function () {
        this.cultures = new Cultures();
        this.log("loaded ".concat(this.cultures.length, " cultures"));
    };
    FactionManager.prototype.createDefaultEmpires = function () {
        // indie empire
        var independent = new Empire("I", "Independent", null, null);
        this.addEmpire(independent);
    };
    FactionManager.prototype.assignFamilyHomes = function () {
        if (this.hasFamilyHomesLoaded) {
            return;
        }
        for (var _i = 0, _a = this.families; _i < _a.length; _i++) {
            var family = _a[_i];
            var familyHasNoHome = !family.home;
            if (familyHasNoHome && family.hasRandomHome) {
                var rndCity = globalThis.CITIES.getRandom();
                family.setHome(rndCity);
            }
        }
        this.hasFamilyHomesLoaded = true;
    };
    FactionManager.prototype.loadEmpiresFromTsFile = function () {
        var mapEmpires = new WorldEmpireLoader().load();
        var empiresCount = mapEmpires.length;
        var empireNames = [];
        for (var _i = 0, mapEmpires_1 = mapEmpires; _i < mapEmpires_1.length; _i++) {
            var empire = mapEmpires_1[_i];
            this.addEmpire(empire);
            empireNames.push(empire.name);
        }
        var empiresStr = empireNames.join(", ");
        this.log("".concat(empiresCount, " empires were loaded: ").concat(empiresStr));
    };
    FactionManager.prototype.assignEmpireCapitals = function (loadOnly) {
        if (this.hasEmpireCapitalsAssigned) {
            return;
        }
        this.log("start empire capital assignment");
        // check for FactionManager
        if (!(globalThis.FACTIONS && globalThis.FACTIONS.empires && globalThis.FACTIONS.empires.length > 1)) {
            this.warn("cant assign empire capitals: FactionManager not initialized");
            return;
        }
        var empireCapitalsCount = 0;
        for (var _i = 0, _a = globalThis.CITIES.getAll(); _i < _a.length; _i++) {
            var city = _a[_i];
            var ownerKey = city._factionKey;
            var empiresWithCapital = Object.keys(this._empireCapitalKeys);
            var ownerHasCapital = empiresWithCapital.includes(ownerKey);
            if (!ownerHasCapital) {
                this.assignSingleEmpireCapital(ownerKey, city.key, true);
            }
        }
        // when all capitals are assigned
        var empireListStr = Object.keys(this._empireCapitalKeys).join(", ");
        this.log("capitals for ".concat(empireCapitalsCount, " empires assigned: ").concat(empireListStr));
        this.hasEmpireCapitalsAssigned = true;
        // inform other managers
        globalThis.ARMIES.onEmpireCapitalsAssigned();
    };
    FactionManager.prototype.assignSingleEmpireCapital = function (empireKey, capitalCityKey, loadOnly) {
        this._empireCapitalKeys[empireKey] = capitalCityKey;
        var empireObj = this.getEmpireByKey(empireKey);
        var setToFaction = empireObj && !loadOnly;
        if (setToFaction) {
            var cityObj = globalThis.CITIES.getByKey(capitalCityKey);
            if (cityObj) {
                empireObj.home = cityObj;
                this._empireCapitalsAssignedCount += 1;
                return true;
            }
        }
        return false;
    };
    FactionManager.prototype.getEmpireCapitalKey = function (empireKey) {
        var empiresWithCapitalKeys = Object.keys(this._empireCapitalKeys);
        if (!empiresWithCapitalKeys.includes(empireKey)) {
            return;
        }
        return this._empireCapitalKeys[empireKey];
    };
    /* STARTING ARMIES */
    FactionManager.prototype.createStartingArmies = function () {
        if (this.hasStartingArmiesCreated) {
            return;
        }
        this.log("placing starting armies for ".concat(this.allFactions.length, " factions.."));
        var armiesCreated = 0;
        for (var _i = 0, _a = this.allFactions; _i < _a.length; _i++) {
            var faction = _a[_i];
            var amount = RandomInt.below(MAX_STARTING_ARMIES);
            for (var index = 0; index < amount; index++) {
                var rndIsPatrol = RandomDice.roll(2);
                faction.createNewArmy(rndIsPatrol);
                armiesCreated += 1;
            }
        }
        this.hasStartingArmiesCreated = true;
        this.log("created ".concat(armiesCreated, " starting armies"));
    };
    FactionManager.prototype.nextFactionNum = function () {
        this.lastFactionNum += 1;
        return this.lastFactionNum;
    };
    /* INTERNAL ON EVENT METHODS */
    FactionManager.prototype.onCitiesLoaded = function () {
        // assign family home & empire capitals
        this.assignFamilyHomes();
        this.assignEmpireCapitals();
        // place starting armies
        this.createStartingArmies();
    };
    FactionManager.prototype.onNameListLoaded = function () {
        this.assignFamilyHomes();
    };
    FactionManager.prototype.onUnitTypesInitialized = function () {
        // this.assignEmpireCapitals();
        // this.createStartingArmies();
    };
    /* GAME EVENT LISTENING */
    FactionManager.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        // gear initialized?
    };
    return FactionManager;
}(AbstractManager));
export { FactionManager };
/* == CONSTANTS == */
var MAX_STARTING_ARMIES = 4;
var RND_FAMILY_COUNT = 5;
//# sourceMappingURL=faction-manager.js.map