var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as EV from "../game-events";
// gui
import { ControllerButtons } from "../gui/menus/controller-buttons";
import { AbstractGuiContainer } from "../gui/common/layouts";
import { DomNode, DomNodes } from "../gui/elements/controls-dom";
import * as GUI_CONST from '../gui/common/constants';
// gui menues
import { TopBarMenu } from '../gui/menus/top-bar';
import { CircularButton } from "../gui/menus/circular-button";
// babylon
import * as GUI from 'babylonjs-gui';
var GUI_TREE_DEPTH = 5;
var GuiManager = /** @class */ (function (_super) {
    __extends(GuiManager, _super);
    function GuiManager() {
        var _this = _super.call(this, "GuiManager") || this;
        // menu controls
        _this.menues = [];
        _this._createDomRoot();
        _this.log("created");
        return _this;
    }
    GuiManager.prototype._createDomRoot = function () {
        // create global DOM_NODES array
        globalThis.DOM_NODES = new DomNodes();
        // create root node
        var domRoot = new DomNode("root");
        globalThis.DOM_NODES.registerNode(domRoot);
        this.log("DOM Root created: ".concat(domRoot));
    };
    GuiManager.prototype.init = function () {
        this.log("building; doing nothing yet");
    };
    GuiManager.prototype.buildAll = function () {
    };
    GuiManager.prototype.getControlById = function (controlId) {
        if (!this.advTexture) {
            this.error("cant get control \"".concat(controlId, "\"; AdvTexture not found"));
            return;
        }
        for (var _i = 0, _a = this.advTexture.getDescendants(); _i < _a.length; _i++) {
            var control = _a[_i];
            var hasSameNameOrId = control.name == controlId;
            if (hasSameNameOrId) {
                return control;
            }
        }
        return;
    };
    GuiManager.prototype.showDialog = function (dialog, forceClose) {
        this.log("showing dialog ".concat(dialog.id, " .."));
        this.closeCurrentDialog(forceClose);
        globalThis.DIALOG = dialog;
        globalThis.DIALOG.build();
    };
    GuiManager.prototype.closeCurrentDialog = function (force) {
        if (globalThis.DIALOG) {
            if (globalThis.DIALOG.isBlocking) {
                if (!force) {
                    this.warn("cant close current dialog ".concat(globalThis.DIALOG.id, "; is blocked"));
                    return;
                }
            }
            globalThis.DIALOG.close();
        }
    };
    /* called from after app assets are loaded */
    GuiManager.prototype.build = function () {
        var scene = globalThis.SCENE;
        this.log("creating babylon adv texture with scene ".concat(scene, ".."));
        this.advTexture = GUI.AdvancedDynamicTexture.CreateFullscreenUI("UI_ROOT", true, scene);
        return this.advTexture;
    };
    GuiManager.prototype.showStartingMenues = function () {
        this.log("showing menues");
        this._createTopBar();
        this._createController();
        this._createCircularButtons();
        this._createBottomMenues();
    };
    /* TOP BAR */
    GuiManager.prototype._createTopBar = function () {
        this.topBar = new TopBarMenu();
        this.menues.push(this.topBar);
        this.advTexture.addControl(this.topBar);
        this.updateTopBar();
    };
    /* create the gameboy like controller buttons to the bottom right */
    GuiManager.prototype._createController = function () {
        this.controllerBtns = new ControllerButtons();
        this.advTexture.addControl(this.controllerBtns);
    };
    GuiManager.prototype._createCircularButtons = function () {
        // add next turn button
        var nextTurnBtn = new CircularButton("turn", GUI_CONST.ALIGN_H_RIGHT);
        this.advTexture.addControl(nextTurnBtn);
        var buildBtn = new CircularButton("build", GUI_CONST.ALIGN_H_LEFT);
        this.advTexture.addControl(buildBtn);
    };
    GuiManager.prototype.onPlayerMoved = function () {
        this.updateTopBar();
    };
    GuiManager.prototype.updateTopBar = function () {
        this.topBar.updateAll();
    };
    /* BOTTOM ACTIONS */
    GuiManager.prototype._createBottomMenues = function () {
    };
    GuiManager.prototype.playSound = function (soundName) {
    };
    /* EVENT LISTENING */
    GuiManager.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        if (ev.id == EV.ID_GUI_ELEM_CLICKED) {
            var elemId = ev.properties.getStr("elemId");
            this.onElementClicked(elemId);
        }
    };
    GuiManager.prototype.onElementClicked = function (elemId) {
        this.log("element \"".concat(elemId, "\" clicked!"));
        // trigger onElementClicked in every menu
        for (var _i = 0, _a = this.menues; _i < _a.length; _i++) {
            var menu = _a[_i];
            menu.onElementClicked(elemId);
        }
        // .. & current dialog
        if (globalThis.DIALOG) {
            globalThis.DIALOG.onElementClicked(elemId);
        }
        return false;
    };
    return GuiManager;
}(AbstractGuiContainer));
export { GuiManager };
// == CONSTANTS == */
// MENUES
export var MENU_TOP_ITEM_WIDTH = "150px";
//# sourceMappingURL=gui-manager.js.map