var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// world
import { WorldSelection } from "../world/selection";
import { WorldPos } from "../world/pos";
import { TerrainTypes } from "../world/tile/tile-terrain";
// manager
import { AbstractManager } from "./abstract-manager";
import { CityDialog } from "../gui/dialogs/city-dialog";
import { WorldTools } from "../../helpers/world-tools";
import { SingleArmyDialog } from "../gui/dialogs/army-dialog";
var SelectionManager = /** @class */ (function (_super) {
    __extends(SelectionManager, _super);
    function SelectionManager() {
        var _this = _super.call(this, "Selection") || this;
        _this.source = new WorldSelection(null);
        _this._cursorPos = new WorldPos(20, 20);
        return _this;
    }
    SelectionManager.prototype.setSourceToPlayer = function () {
        if (globalThis.PLAYER) {
            this.source.setToPlayer();
            var playerPosStr = globalThis.PLAYER.pos.asString();
            this.log("source set to player at ".concat(playerPosStr));
            return;
        }
        this.warn("cant set source to player: Player not found");
    };
    SelectionManager.prototype.moveCursorTo = function (worldPos) {
        this._cursorPos = worldPos;
        var tile = globalThis.MAP.tileAtPos(worldPos);
        var tilePosVec3 = WorldTools.worldToScenePos(tile.pos);
        // update cursor pos
        if (tile && this.cursorMesh) {
            var offsetY = TerrainTypes.heightOf(tile.terrainChar) / 2;
            this.cursorMesh.position.set(tilePosVec3.x, offsetY, tilePosVec3.z);
        }
    };
    SelectionManager.prototype.movePlayerToCursor = function () {
        globalThis.PLAYER_M.movePlayerTo(this._cursorPos);
    };
    /* ON ENTITY CLICK */
    SelectionManager.prototype.onCityClicked = function (city) {
        this.log("click on city ".concat(city.name));
        // set as selection source
        this.source.setToCity(city);
        this.onPosClick(city.pos);
        // show city dialog
        var cityDiag = new CityDialog(city);
        globalThis.GUI.showDialog(cityDiag);
    };
    SelectionManager.prototype.onPlayerClicked = function () {
        this.onPosClick(globalThis.PLAYER.pos);
        globalThis.PLAYER_M.onPlayerClicked();
    };
    SelectionManager.prototype.onTileClicked = function (tile) {
        this.log("clicked on tile ".concat(tile.terrainChar, " at ").concat(tile.pos.asString()));
        this.onPosClick(tile.pos);
        this.moveCursorTo(tile.pos);
    };
    SelectionManager.prototype.onCursorClicked = function () {
        var cursorPosStr = this._cursorPos.asString();
        this.log("clicked on cursor at ".concat(cursorPosStr, ": initialize movement check"));
        // check for possible movement
        this.movePlayerToCursor();
    };
    SelectionManager.prototype.onPosClick = function (clickedPos) {
        this._clickedPos = clickedPos;
    };
    SelectionManager.prototype.onPlayerMeshPlaced = function () {
        this.setSourceToPlayer();
    };
    SelectionManager.prototype.onArmyClicked = function (army) {
        this.log("click on army \"".concat(army.name, "\" of ").concat(army._ownerKey));
        // show army dialog
        var armyDiag = new SingleArmyDialog(army);
        globalThis.GUI.showDialog(armyDiag);
    };
    /* EVENT HANDLERS FROM OTHER MANAGERS */
    SelectionManager.prototype.onMeshClicked = function (mesh) {
        // CITY click
        if (mesh.id.startsWith("city")) {
            var clickedCity = SelectionManager.GetCityByMesh(mesh);
            this.onCityClicked(clickedCity);
            // ARMY click
        }
        else if (mesh.id.startsWith("army")) {
            var clickedArmy = SelectionManager.GetArmyByMesh(mesh);
            this.onArmyClicked(clickedArmy);
            // PLAYER caravan click
        }
        else if (mesh.id.startsWith("player")) {
            this.onPlayerClicked();
            // TILE click on map
        }
        else if (mesh.id.startsWith("tile")) {
            var tile = SelectionManager.GetTileByMesh(mesh);
            if (tile) {
                this.onTileClicked(tile);
            }
            // CURSOR click
        }
        else if (mesh.id.startsWith("cursor")) {
            this.onCursorClicked();
        }
    };
    Object.defineProperty(SelectionManager.prototype, "cursorMesh", {
        /* SELECTION MANAGER GETTERS */
        get: function () {
            return globalThis.WORLD.cursorMesh;
        },
        enumerable: false,
        configurable: true
    });
    /* STATIC METHODS */
    SelectionManager.GetTileByMesh = function (tileMesh) {
        var tilePosStr = tileMesh.id.split("-")[2];
        return globalThis.MAP.tileAtPos(WorldPos.FromString(tilePosStr, true));
    };
    SelectionManager.GetCityByMesh = function (cityMesh) {
        var cityKey = cityMesh.name.split("-")[1];
        return globalThis.CITIES.getByKey(cityKey);
    };
    SelectionManager.GetArmyByMesh = function (armyMesh) {
        var armyNum = parseInt(armyMesh.name.split("-")[1]);
        return globalThis.ARMIES.ByNum(armyNum);
    };
    SelectionManager.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
    };
    return SelectionManager;
}(AbstractManager));
export { SelectionManager };
//# sourceMappingURL=selection-manager.js.map