var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { BaseEventListener } from "../../common/events";
import { GameOptions } from "../game-options";
import { Loggable } from "../../common/log";
import { RandomElement } from "../../helpers/random";
// resources
/* CHANGE USED MAP HERE */
import * as MAPDATA from "../../../res/worlds/world_120_1600";
// world
import { WorldPos } from "../world/pos";
import { WorldTile } from "../world/tile/tile";
import { LandscapeFeature } from "../world/feature";
// models
import { City } from "../entities/city/city";
import { Empire } from "../entities/faction/empire";
import { Properties } from "../../common/properties";
import { TradeResource } from "../entities/trade/resource";
import { Family } from "../entities/faction/family";
var WorldTilesLoader = /** @class */ (function (_super) {
    __extends(WorldTilesLoader, _super);
    function WorldTilesLoader() {
        return _super.call(this, "WorldTileLoader") || this;
    }
    WorldTilesLoader.prototype.load = function () {
        var tileRows = [];
        var createdTilesCount = 0;
        // for row
        for (var z = 0; z < MAPDATA.TILE_ROWS.length; z++) {
            var terrRow = MAPDATA.TILE_ROWS[z];
            var tileRow = [];
            for (var x = 0; x < terrRow.length; x++) {
                // find terrain char in array
                var pos = new WorldPos(x, z);
                var terrainChar = terrRow[pos.x];
                // create WorldTile & increase counter
                var worldTile = this.makeWorldTile(terrainChar, pos);
                createdTilesCount += 1;
                tileRow.push(worldTile);
            } // for x end
            tileRows.push(tileRow);
        } // for z end
        this.log("".concat(createdTilesCount, " WorldTile objects loaded"));
        return tileRows;
    };
    /* returns WorldTile object with given pos & terrain */
    WorldTilesLoader.prototype.makeWorldTile = function (terrainChar, pos) {
        var tile = new WorldTile(pos, terrainChar);
        // check whether tile is overwritten
        var terrainOverride = this.getTerrainOverwriteAtPos(pos);
        if (terrainOverride) {
            tile.terrainChar = terrainOverride;
        }
        return tile;
    };
    WorldTilesLoader.prototype.getTerrainOverwriteAtPos = function (pos) {
        for (var _i = 0, _a = MAPDATA.TILE_OVERRIDE; _i < _a.length; _i++) {
            var tileOverwriteStr = _a[_i];
            var tileOverrideData = tileOverwriteStr.split(" ");
            var tileOverridePos = WorldPos.FromString(tileOverwriteStr);
            if (tileOverridePos.equals(pos)) {
                var tileOverwriteTerrain = tileOverrideData[2];
                return tileOverwriteTerrain;
            }
        }
        return null;
    };
    return WorldTilesLoader;
}(BaseEventListener));
export { WorldTilesLoader };
/* == LOADER: LANDSCAPE FEATURE == */
var WorldLandscapeLoader = /** @class */ (function (_super) {
    __extends(WorldLandscapeLoader, _super);
    function WorldLandscapeLoader() {
        return _super.call(this, "LandscapeLoader") || this;
    }
    WorldLandscapeLoader.prototype.load = function () {
        var features = [];
        for (var _i = 0, _a = MAPDATA.LANDSCAPE; _i < _a.length; _i++) {
            var featureStr = _a[_i];
            var feature = LandscapeFeature.FromString(featureStr);
            features.push(feature);
        }
        return features;
    };
    return WorldLandscapeLoader;
}(BaseEventListener));
export { WorldLandscapeLoader };
/* == GENERATOR: CITIES == */
var CityGenerator = /** @class */ (function (_super) {
    __extends(CityGenerator, _super);
    function CityGenerator() {
        var _this = _super.call(this, "CityGenerator") || this;
        _this.cities = [];
        return _this;
    }
    CityGenerator.prototype.generate = function () {
        var maxCities = globalThis.OPTIONS.get(GameOptions.MAP_MAXCITIES);
        var generatedCities = [];
        for (var cityNum = 0; cityNum < maxCities; cityNum++) {
            var newCity = this.generateSingleCity(cityNum);
            generatedCities.push(newCity);
        }
        return generatedCities;
    };
    CityGenerator.prototype.generateSingleCity = function (cityNum) {
        var generatedCity = City.MakeRandom();
        generatedCity.num = cityNum;
        return generatedCity;
    };
    return CityGenerator;
}(BaseEventListener));
export { CityGenerator };
/* == LOADER: CITIES == */
var CityLoader = /** @class */ (function (_super) {
    __extends(CityLoader, _super);
    function CityLoader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CityLoader.load = function () {
        var cities = [];
        for (var _i = 0, _a = MAPDATA.CITIES; _i < _a.length; _i++) {
            var cityValues = _a[_i];
            var loadedCity = City.FromValueArray(cityValues);
            cities.push(loadedCity);
        }
        return cities;
    };
    return CityLoader;
}(Loggable));
export { CityLoader };
/* == LOADER: RND START POS == */
var RandomStartPosLoader = /** @class */ (function (_super) {
    __extends(RandomStartPosLoader, _super);
    function RandomStartPosLoader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RandomStartPosLoader.load = function () {
        var positions = MAPDATA.RND_START_POS;
        var rndPosStr = RandomElement.of(positions);
        return WorldPos.FromString(rndPosStr);
    };
    return RandomStartPosLoader;
}(Loggable));
export { RandomStartPosLoader };
/* == LOADER: MAP EMPIRES == */
var WorldEmpireLoader = /** @class */ (function (_super) {
    __extends(WorldEmpireLoader, _super);
    function WorldEmpireLoader() {
        return _super.call(this, 'EmpireLoader') || this;
    }
    WorldEmpireLoader.prototype.load = function () {
        var empires = [];
        for (var _i = 0, _a = MAPDATA.CUSTOM_EMPIRES; _i < _a.length; _i++) {
            var empireJson = _a[_i];
            if (!empireJson["key"]) {
                this.warn("cant load empire ".concat(empireJson.toString(), " has no key property}"));
                continue;
            }
            var empireKey = empireJson["key"];
            var empireName = empireJson["name"];
            var empire = new Empire(empireKey, empireName);
            // set empire flag data
            var flagStr = empireJson["flag"];
            empire.flagData = flagStr;
            empire.updateFlag();
            empires.push(empire);
        }
        return empires;
    };
    return WorldEmpireLoader;
}(BaseEventListener));
export { WorldEmpireLoader };
/* == LOADER: CUSTOM FAMILIES == */
var CustomFamiliesLoader = /** @class */ (function (_super) {
    __extends(CustomFamiliesLoader, _super);
    function CustomFamiliesLoader() {
        return _super.call(this, "CustomFamilyLoader") || this;
    }
    CustomFamiliesLoader.prototype.load = function () {
        if (!Object.keys(MAPDATA).includes("CUSTOM_FAMILIES")) {
            this.warn("cant load custom families; none in map file");
            return;
        }
        var customFamilies = [];
        for (var _i = 0, _a = MAPDATA["CUSTOM_FAMILIES"]; _i < _a.length; _i++) {
            var familyJson = _a[_i];
            var name_1 = familyJson["name"];
            var family = new Family(name_1, null, false);
            // set home key but load city later
            family._homeCityKey = familyJson["home"];
            // set optional properties
            family.customLanguage = familyJson["lang"];
            family.customStartPos = WorldPos.FromString(familyJson["customStartPos"]);
            customFamilies.push(family);
        }
        return customFamilies;
    };
    return CustomFamiliesLoader;
}(BaseEventListener));
export { CustomFamiliesLoader };
/* LOADER: RESOURCES */
var SpecialResLoader = /** @class */ (function (_super) {
    __extends(SpecialResLoader, _super);
    function SpecialResLoader() {
        return _super.call(this, 'SpecialResLoader') || this;
    }
    SpecialResLoader.prototype.load = function () {
        var specialResources = [];
        if (MAPDATA["SPECIAL_RESOURCES"]) {
            var specialResJson = MAPDATA["SPECIAL_RESOURCES"];
            for (var _i = 0, specialResJson_1 = specialResJson; _i < specialResJson_1.length; _i++) {
                var singleResJson = specialResJson_1[_i];
                var tradeRes = TradeResource.FromJson(singleResJson);
                specialResources.push(tradeRes);
            }
        }
        return specialResources;
    };
    return SpecialResLoader;
}(BaseEventListener));
export { SpecialResLoader };
/* LOADER: MAP PROPERTIES == */
var WorldPropertiesLoader = /** @class */ (function (_super) {
    __extends(WorldPropertiesLoader, _super);
    function WorldPropertiesLoader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WorldPropertiesLoader.load = function () {
        var mapProperties = new Properties("map");
        // start & end year
        mapProperties.addInteger("startYear", MAPDATA.STARTING_YEAR);
        mapProperties.addInteger("endYear", MAPDATA.END_YEAR);
        // seasons per turn
        mapProperties.addInteger("seasonsPerYear", MAPDATA.SEASONS_PER_YEAR);
        return mapProperties;
    };
    return WorldPropertiesLoader;
}(BaseEventListener));
export { WorldPropertiesLoader };
//# sourceMappingURL=world-loaders.js.map