var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Properties } from "../../common/properties";
import { DawnbringerPalette } from "../../common/colors";
import * as EV from "../game-events";
// entities
import { WorldEventEntity } from "../../common/entity";
import { Building } from "../entities/building/building";
import { TradeConnection } from "../entities/trade/connection";
import { StoredResource } from "../entities/trade/resource-storage";
import { TradeHistory } from "../entities/trade/trade-history";
/* == ABSTRACT LOCATION MODEL == */
var AbstractLocation = /** @class */ (function (_super) {
    __extends(AbstractLocation, _super);
    function AbstractLocation(num, pos, name) {
        var _this = _super.call(this, num, pos) || this;
        // basic properties
        _this.key = null;
        _this.level = 1;
        _this.properties = new Properties();
        // flags
        _this.canTrade = false;
        // world properties
        _this.neighborTiles = [];
        _this.roads = [];
        // buildings
        _this.buildings = [];
        _this.name = name;
        _this.randomizeStartingMoney();
        return _this;
    }
    AbstractLocation.prototype.randomizeStartingMoney = function () {
        this.properties.addInteger(PROPERTY_MONEY, -1, DawnbringerPalette.GOLDENFIZZ, 1000);
    };
    AbstractLocation.prototype.equals = function (otherCity) {
        return this.name == otherCity.name;
    };
    Object.defineProperty(AbstractLocation.prototype, "money", {
        get: function () { return this.properties.getNum(PROPERTY_MONEY); },
        set: function (value) { this.properties.getProperty(PROPERTY_MONEY).value = value; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(AbstractLocation.prototype, "tile", {
        /* TILE */
        get: function () {
            var tileAt = globalThis.MAP.tileAtPos(this.pos);
            return tileAt;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(AbstractLocation.prototype, "neighborTerrains", {
        get: function () { return this.tile.neighborTerrains; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(AbstractLocation.prototype, "terrainId", {
        get: function () { return this.tile.terrainChar; },
        enumerable: false,
        configurable: true
    });
    /* BUILDINGS */
    AbstractLocation.prototype.addBuilding = function (bldType, level) {
        // this.log(`adding building ${bldType}`);
        if (!level) {
            level = 0;
        }
        var bldInstance = new Building(bldType, "", level);
        this.buildings.push(bldInstance);
        // fire ConstructionFinishedEvent
        globalThis.EVENTS.fireEvent(new EV.ConstructionFinishedEvent(this, bldType, level));
    };
    return AbstractLocation;
}(WorldEventEntity));
export { AbstractLocation };
var ConnectedLocation = /** @class */ (function (_super) {
    __extends(ConnectedLocation, _super);
    function ConnectedLocation(num, pos, name) {
        var _this = _super.call(this, num, pos, name) || this;
        _this.connections = [];
        return _this;
    }
    ConnectedLocation.prototype.getConnectionTo = function (target) {
        for (var _i = 0, _a = this.connections; _i < _a.length; _i++) {
            var connection = _a[_i];
            var isToTarget = connection.target.equals(target);
            if (isToTarget) {
                return connection;
            }
        }
        return null;
    };
    ConnectedLocation.prototype.addConnectionTo = function (targetLocation, addMirrored) {
        var connection = new TradeConnection(this, targetLocation);
        this.connections.push(connection);
        // this.tradeNode.linkedMarkets.push(targetLocation.tradeNode);
        if (addMirrored) {
            targetLocation.addConnectionTo(this, false);
        }
        return connection;
    };
    ConnectedLocation.prototype.getConnectedLocations = function () {
        var connected = [];
        for (var _i = 0, _a = this.connections; _i < _a.length; _i++) {
            var connection = _a[_i];
            if (connection.target.key != this.key) {
                connected.push(connection.target);
            }
        } // end of connections loop
        return connected;
    };
    return ConnectedLocation;
}(AbstractLocation));
export { ConnectedLocation };
/* == OWNED LOCATION MODEL == */
var OwnedLocation = /** @class */ (function (_super) {
    __extends(OwnedLocation, _super);
    function OwnedLocation(num, pos, name, owner) {
        var _this = _super.call(this, num, pos, name) || this;
        _this.owner = null;
        if (owner) {
            _this.owner = owner;
        }
        return _this;
    }
    OwnedLocation.prototype.getOwner = function () {
        return this.owner;
    };
    Object.defineProperty(OwnedLocation.prototype, "ownerEmpire", {
        get: function () {
            if (!globalThis.FACTIONS) {
                this.warn("cant get empire of \"".concat(this.name, "\"; FactionManager not found"));
                return;
            }
            return globalThis.FACTIONS.getEmpireByKey(this._factionKey);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(OwnedLocation.prototype, "ownerFamily", {
        get: function () {
            if (!globalThis.FACTIONS) {
                this.warn("cant get family of \"".concat(this.name, "\"; FactionManager not found"));
                return;
            }
            if (this._familyNum) {
                return globalThis.FACTIONS.getFamilyByNum(this._familyNum);
            }
            return;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(OwnedLocation.prototype, "ownerName", {
        get: function () {
            if (this.ownerFamily) {
                return this.ownerFamily.name;
            }
            if (this.ownerEmpire) {
                return this.ownerEmpire.name;
            }
            return "Independent";
        },
        enumerable: false,
        configurable: true
    });
    return OwnedLocation;
}(ConnectedLocation));
export { OwnedLocation };
/* == TRADE LOCATION MODELS == */
var TradeLocation = /** @class */ (function (_super) {
    __extends(TradeLocation, _super);
    function TradeLocation(num, pos, name, owner) {
        var _this = _super.call(this, num, pos, name, owner) || this;
        _this.canTrade = true;
        _this.tradeHistory = new TradeHistory(_this);
        return _this;
    }
    TradeLocation.prototype.addTradeConnectionTo = function (targetLocation, addMirrored) {
        var connection = _super.prototype.addConnectionTo.call(this, targetLocation, addMirrored);
        this.tradeNode.linked.push(targetLocation.tradeNode);
        return connection;
    };
    TradeLocation.prototype.tradeIsExport = function (trade) { return this.tradeNode.tradeIsExport(trade); };
    /* RESOURCES */
    TradeLocation.prototype.canAfford = function (resId, amount) {
        var singlePrice = this.tradeNode.priceOf(resId);
        var totalPrice = amount * singlePrice;
        if (this.money >= totalPrice) {
            return true;
        }
        return false;
    };
    TradeLocation.prototype.getStoredResources = function () {
        var actuallyStored = [];
        for (var _i = 0, _a = this.tradeNode.asStoredResources(); _i < _a.length; _i++) {
            var storedRes = _a[_i];
            var resAmount = this.tradeNode.amountOf(storedRes.resId);
            if (resAmount > 0) {
                var resPrice = this.tradeNode.priceOf(storedRes.resId);
                var stored = new StoredResource(storedRes.resId, resAmount, resPrice);
                actuallyStored.push(stored);
            }
        }
        return actuallyStored;
    };
    TradeLocation.prototype.canProduce = function (resId) {
        var resObj = globalThis.TRADE.GetRes(resId);
        // can build on city tile terrain?
        if (resObj.canBuildProdBldOn(this.terrainId)) {
            return true;
        }
        // can build on neighboring tile terrain?
        for (var _i = 0, _a = this.neighborTerrains; _i < _a.length; _i++) {
            var terrainChar = _a[_i];
            if (resObj.canBuildProdBldOn(terrainChar)) {
                return true;
            }
        }
        return false;
    };
    TradeLocation.prototype.produceResource = function (resId, amount) {
        this.tradeNode.add(resId, amount);
    };
    /* CARAVAMS */
    TradeLocation.prototype.formCaravan = function (target, resId, teleport) {
        var resName = globalThis.TRADE.getResName(resId);
        this.log("forming ".concat(resName, " caravan to ").concat(target.name));
    };
    /* CONNECTIONS */
    TradeLocation.prototype.getConnectedTradeLocations = function () {
        var tradeLocations = [];
        for (var _i = 0, _a = this.getConnectedLocations(); _i < _a.length; _i++) {
            var connected = _a[_i];
            if (connected instanceof TradeLocation) {
                tradeLocations.push(connected);
            }
        }
        return tradeLocations;
    };
    return TradeLocation;
}(OwnedLocation));
export { TradeLocation };
/* == CONSTANTS == */
export var PROPERTY_MONEY = "money";
//# sourceMappingURL=location.js.map