var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { DawnbringerPalette, GrayTone } from '../../common/colors';
import { GameOptions } from '../game-options';
import { Loggable } from '../../common/log';
// world
import { TileMaterials, WorldTile } from './tile/tile';
import { TerrainTypes } from './tile/tile-terrain';
import { WorldTilesLoader, WorldLandscapeLoader, RandomStartPosLoader, WorldPropertiesLoader, SpecialResLoader } from '../managers/world-loaders';
import { WorldPos, Size } from './pos';
// babylon
import * as BABYLON from 'babylonjs';
/* == WORLD MAP MODEL == */
var WorldMap = /** @class */ (function (_super) {
    __extends(WorldMap, _super);
    function WorldMap(name, size) {
        var _this = _super.call(this, "WorldMap") || this;
        _this.size = new Size(100, 100);
        _this.properties = null;
        _this.tiles = [];
        _this.playerStartingPos = null;
        // landscape
        _this.features = [];
        // materials
        _this.mats = null;
        _this.name = name;
        _this.size = size;
        _this.terrainIds = TerrainTypes.LIST;
        _this.generateOrLoadTiles();
        return _this;
    }
    WorldMap.prototype.generateOrLoadTiles = function () {
        var generate = globalThis.OPTIONS.getBool(GameOptions.GENERATE_LANDSCAPE);
        this.log("loading tiles: random? ".concat(generate));
        if (generate) {
            this._generateTiles();
        }
        else {
            this._loadProperties();
            this._loadSpecialRes();
            this._loadTiles();
            this._loadFeatures();
            this._loadStartingPos();
        }
    };
    WorldMap.prototype._loadProperties = function () {
        this.properties = WorldPropertiesLoader.load();
    };
    WorldMap.prototype._loadStartingPos = function () {
        this.playerStartingPos = RandomStartPosLoader.load();
    };
    WorldMap.prototype._generateTiles = function () {
        this.log("tilemap: creating matrix");
        var generatedTilesCount = 0;
        for (var z = 0; z < this.size.z; z++) {
            var tileRow = [];
            for (var x = 0; x < this.size.x; x++) {
                var terrain = this.getRandomTerrain();
                var tilePos = new WorldPos(x, z);
                var tile = new WorldTile(tilePos, terrain);
                // add tile & increase counter
                tileRow.push(tile);
                generatedTilesCount += 1;
            }
            this.tiles.push(tileRow);
        }
        this.log("".concat(generatedTilesCount, " tiles generated"));
    };
    WorldMap.prototype._loadTiles = function () {
        this.tiles = new WorldTilesLoader().load();
        // reset map size
        this.size = new Size(this.tiles[0].length, this.tiles.length);
        this.log("resized to ".concat(this.size.x, "x").concat(this.size.z));
    };
    WorldMap.prototype._loadFeatures = function () {
        var features = new WorldLandscapeLoader().load();
        this.features = features;
    };
    WorldMap.prototype._loadSpecialRes = function () {
        var specialRes = new SpecialResLoader().load();
        globalThis.SPECIAL_RES = specialRes;
    };
    WorldMap.prototype.getRandomTerrain = function () {
        var landscapeTerrains = TerrainTypes.LANDSCAPE;
        var rndInt = Math.floor(Math.random() * landscapeTerrains.length);
        return landscapeTerrains[rndInt];
    };
    WorldMap.prototype.tileAt = function (x, z) {
        var row = this.tiles[z];
        if (!row) {
            this.warn("cant find row ".concat(z, " in ").concat(this.tiles.length, " tile array rows"));
            return;
        }
        var tileAt = row[x];
        if (!tileAt) {
            console.error("cant get tile at ".concat(x, " ").concat(z));
            return;
        }
        return tileAt;
    };
    WorldMap.prototype.tileAtPos = function (pos) {
        return this.tileAt(pos.x, pos.z);
    };
    // ???
    WorldMap.prototype.createMaterials = function () {
        /* == DEFINE TERRAIN COLORS HERE == */
        this.mats = new TileMaterials(this);
        this.mats.add(TerrainTypes.WATER, this.paletteColor(DawnbringerPalette.CORNFLOWER), new GrayTone(0.5).toBabylon(), 128);
        this.mats.add(TerrainTypes.GRASS, this.paletteColor(DawnbringerPalette.CHRISTI), new GrayTone(0.05).toBabylon(), 128);
        this.mats.add(TerrainTypes.HILL, this.paletteColor(DawnbringerPalette.ROPE), new GrayTone(0.05).toBabylon(), 128);
        this.mats.add(TerrainTypes.FOREST, this.paletteColor(DawnbringerPalette.DELL), new GrayTone(0.05).toBabylon(), 128);
        this.mats.add(TerrainTypes.STONE, this.paletteColor(DawnbringerPalette.SMOKEYASH), new GrayTone(0.01).toBabylon(), 128);
        this.mats.add(TerrainTypes.BARREN, this.paletteColor(DawnbringerPalette.DIMGRAY), new GrayTone(0.01).toBabylon(), 128);
        this.mats.add(TerrainTypes.MOUNTAIN, this.paletteColor(DawnbringerPalette.TOPAZ), new GrayTone(0.01).toBabylon(), 128);
        this.mats.add(TerrainTypes.PEAK, this.paletteColor(DawnbringerPalette.LIGHTSTEEL), new GrayTone(0.01).toBabylon(), 128);
        this.mats.add(TerrainTypes.CAVE, this.paletteColor(DawnbringerPalette.SMOKEYASH), new GrayTone(0.01).toBabylon(), 128);
        this.mats.add(TerrainTypes.EARTH, this.paletteColor(DawnbringerPalette.ROPE));
        this.mats.add(TerrainTypes.DESERT, this.paletteColor(DawnbringerPalette.PANCHO));
        this.mats.add(TerrainTypes.SAVANNAH, this.paletteColor(DawnbringerPalette.RAINFOREST));
        this.mats.add(TerrainTypes.CITY, this.paletteColor(DawnbringerPalette.PANCHO));
        this.mats.add(TerrainTypes.ARCTIC, this.paletteColor(DawnbringerPalette.LIGHTSTEEL));
        this.mats.add(TerrainTypes.BRIDGE, this.paletteColor(DawnbringerPalette.VIKING));
        this.mats.add(TerrainTypes.LAVA, this.paletteColor(DawnbringerPalette.TAHITIGOLD));
        this.log("created ".concat(this.mats.length, " tile materials"));
    };
    WorldMap.prototype.paletteColor = function (rgb) {
        return new BABYLON.Color3(rgb[0], rgb[1], rgb[2]);
    };
    WorldMap.prototype.getMaterial = function (key) {
        return this.mats.get(key);
    };
    Object.defineProperty(WorldMap.prototype, "DEFAULT_SIZE", {
        get: function () {
            return new Size(100, 100);
        },
        enumerable: false,
        configurable: true
    });
    WorldMap.prototype.toString = function () {
        return "<WorldMap \"".concat(this.name, ">");
    };
    WorldMap.prototype.makePos = function (x, z) {
        return new WorldPos(x, z);
    };
    return WorldMap;
}(Loggable));
export { WorldMap };
/* GLOBAL METHODS */
export function tileAt(x, z) {
    return globalThis.MAP.tileAt(x, z);
}
//# sourceMappingURL=map.js.map