var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Loggable } from '../../../common/log';
// world
import { HexDirection, WorldPos } from '../pos';
// babylon
import * as BABYLON from 'babylonjs';
import { TerrainTypes } from './tile-terrain';
/* == TILE MODEL == */
var WorldTile = /** @class */ (function (_super) {
    __extends(WorldTile, _super);
    function WorldTile(pos, terrainChar) {
        var _this = _super.call(this, "WorldTile") || this;
        _this.offset = 0;
        // entities
        _this.connections = [];
        _this.roads = [];
        _this.city = null;
        // meshes
        _this.meshBlueprint = null;
        _this.meshInstance = null;
        _this.pos = pos;
        _this.terrainChar = terrainChar;
        return _this;
    }
    WorldTile.prototype.toString = function () {
        var posStr = "".concat(this.pos.x, "|").concat(this.pos.z);
        return "<WorldTile \"".concat(this.terrainChar, "\" pos=").concat(posStr, ">");
    };
    Object.defineProperty(WorldTile, "HEXSIZE", {
        /* GETTERS */
        get: function () { return 3; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WorldTile.prototype, "isOnLand", {
        get: function () {
            return !(this.terrainChar === TerrainTypes.WATER);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WorldTile.prototype, "meshName", {
        get: function () {
            return "tile-".concat(this.terrainChar, "-").concat(this.pos.asString());
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WorldTile.prototype, "isOnRoad", {
        get: function () { return this.roads.length > 0; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WorldTile.prototype, "neighborTerrains", {
        get: function () {
            var neighbTerrains = [];
            var neighbDirs = HexDirection.ALL;
            for (var _i = 0, neighbDirs_1 = neighbDirs; _i < neighbDirs_1.length; _i++) {
                var dirStr = neighbDirs_1[_i];
                var dirObj = new HexDirection(dirStr);
                var relPos = dirObj.relPos;
                var absPos = new WorldPos(this.pos.x + relPos.x, this.pos.z + relPos.z);
                var neighbTile = globalThis.MAP.tileAtPos(absPos);
                var neighbTileTerrain = neighbTile.terrainChar;
                neighbTerrains.push(neighbTileTerrain);
            }
            return neighbTerrains;
        },
        enumerable: false,
        configurable: true
    });
    return WorldTile;
}(Loggable));
export { WorldTile };
/* TILE MATERIALS */
var TileMaterials = /** @class */ (function () {
    function TileMaterials(map) {
        this._keys = [];
        this._mats = [];
        this._map = map;
    }
    TileMaterials.prototype.add = function (name, diffuseColor, specularColor, specularPower) {
        var mat = new BABYLON.StandardMaterial(name, globalThis.SCENE);
        mat.diffuseColor = diffuseColor;
        if (specularColor) {
            mat.specularColor = specularColor;
            if (specularPower) {
                mat.specularPower = specularPower;
            }
        }
        this._keys.push(name);
        this._mats.push(mat);
        // console.log(`material ${name} created: ${mat}`);
    };
    TileMaterials.prototype.get = function (key) {
        var matIndex = this._keys.indexOf(key);
        var mat = this._mats[matIndex];
        return mat;
    };
    Object.defineProperty(TileMaterials.prototype, "length", {
        get: function () {
            return this._keys.length;
        },
        enumerable: false,
        configurable: true
    });
    return TileMaterials;
}());
export { TileMaterials };
//# sourceMappingURL=tile.js.map