import {Character} from "./character";

export interface ICharacterAction {
    character: Character;
    typeNum: number;
    actionKey: string;
    actionValue: number;
    actionData: string;
    // flags
    canBeExecuted: boolean;
    // methods
    onExecute(charAction: ICharacterAction): boolean;
}

/**
 * A possible action to take for a character for one turn. An example could be taking a rest to heal or
 * attacking an enemy character that is residing in the same tile.
 *
  */

export class AbstractCharacterAction implements ICharacterAction {
    character: Character = null;
    typeNum: number;
    actionKey: string;
    actionValue: number = -1;
    actionData: string = null;

    constructor(actionKey: string, typeNum: number, actionValue?: number, actionData?: string, character?: Character) {
        this.typeNum = typeNum;
        this.actionKey = actionKey;
        // optional
        if (actionValue && actionValue >0) { this.actionValue = actionValue; }
        if (actionData && actionData.length > 0) { this.actionData = actionData; }
        if (character) { this.character = character; }
    }

    get canBeExecuted(): boolean {
        return false;
    }

    onExecute(this): boolean {
        return false;
    }
}

export class CharacterActionType {

    static get MOVEMENT() { return 1; }
    static get COURT() { return 2; }
    static get FIGHT() { return 3; }
    static get TRADE() { return 4; }
    static get PLOT() { return 5; }
    static get OTHER() { return 6; }
}