// entities
import { Unit } from "../unit/unit";
import { City } from "../city/city";
import { IFaction, AbstractFaction } from "./faction";
import {IOnTurnEntity, IUniqueInstance} from "../../../common/entity";
// helpers
import { RandomElement } from "../../../helpers/random";
import { WorldPos } from "../../world/pos";
import { FACTION_TYPES, TYPE_FAMILY } from "./faction-type";


/* == FAMILY INTERFACE == */
export interface IFamily extends IFaction {
    hasRandomHome: boolean;
    isAi: boolean;
    manorCityNums: Array<number>;
    // methods
    setNum(familyNum: number);
}

/* == FAMILY MODEL == */
export class Family extends AbstractFaction implements IFamily {
    num: number;
    name: string;
    units: Array<Unit> = [];
    // cities
    _homeCity: City;
    _homeCityKey: string;
    manorCityNums: Array<number> = [];
    // flags
    hasRandomHome = false;
    isAi: boolean = true;
    customLanguage: string;
    customStartPos: WorldPos;

    constructor(name: string, home: City, isPlayer?: boolean) {
        super(null, name, home);
        this.factionType = FACTION_TYPES[TYPE_FAMILY];
        // optional
        if (! name) { this.name = Family.randomName(); }
        if ( isPlayer ) { this.isAi = false; }
    }

    setNum(familyNum: number) {
        this.num = familyNum;
        this.key = "F" + familyNum;
    }

    setHome(city: City) {
        this._homeCity = city;
        // this.home = city;
        this.setRandomNameByHome();
    }

    /* RANDOM GENERATION */
    static randomName(): string {
        return RandomElement.of(Family.SURNAMES) as string;
    }

    setRandomNameByHome() {
        if (! this.homeCity) { return; }
        if (! globalThis.CHARS.hasNameListLoaded ) {
            this.error(`cant set random name from home; Char name list not found`); return;
        }
        this.name = globalThis.CHARS.getRandomFamilyName(this.homeCity.language);
    }

    /* returns a new randomly generated Family with random name & home */
    static random(): Family {
        let rndName: string = Family.randomName();
        let rndHome: City = globalThis.CITIES.getRandom();
        let family: Family = new Family(rndName, rndHome);
        return family;
    }

    buildManorIn(city: City) {
        this.manorCityNums.push( city.num );
    }

    /* GETTERS */
    get homeCity(): City {
        if (this._homeCity) { return this._homeCity; }
        else if (this._homeCityKey && globalThis.CITIES) {
            return globalThis.CITIES.getByKey(this._homeCityKey);
        }
    }

    /* CONSTANTS */
    static get PREFIX() { return "House"; }
    static get SURNAMES() {
        return ["Valecci", "Du Chapel", "Fugger", "Habsburg", "Rurik", "Qing", "of Cornwall", "Di Bordia"]
    }

    /* LOG */
    toString(): string {
        return `<Family ${this.num} "${this.name}">`;
    }
}

/* == FAMILY CONSTANTS == */
