import { Loggable } from "../../../common/log";
import { TradeNode } from "./trade-node";
import { MarketTrade } from "./market-trade";

/* == INTERFACES == */
export interface ITradeHistoryKeeper {
    tradeHistory: ITradeHistory;
    // methods
    tradeIsExport(trade: MarketTrade): boolean;
}

export interface ITradeHistory {
    parent: ITradeHistoryKeeper;
    trades: MarketTrade[];
    length: number;
    // methods
    add(trade: MarketTrade, isExport?: boolean);
    getLastTrades(amount: number);
}

/* == TRADE HISTORY == */
export class TradeHistory extends Loggable {
    parent: ITradeHistoryKeeper;
    isForConnection: boolean = false;
    trades: MarketTrade[] = [];

    constructor(parent: ITradeHistoryKeeper, isForConnection?: boolean) {
        super("TradeHistory");
        this.parent = parent;
        if (isForConnection) { this.isForConnection = true; }
    }

    add(trade: MarketTrade, isExport?: boolean) {
        if (isExport) { trade.isExport = true; }
        this.trades.push(trade);
    }

    get length(): number {
        return this.trades.length;
    }

    getLastTrades(amount: number) {
        return this.trades.slice(-1 * amount);
    }
}