// common
import { DawnbringerPalette } from "../../../common/colors";
// world
import { WorldPos } from "../../world/pos";
// events
import { IBaseEvent } from "../../../common/events";
import * as EV from "../../game-events";
// entities
import { UnitArmy } from "../../entities/unit/army/army";
// gui
import * as GUI_CONST from '../common/constants';
import { EmptyDialog, IDialog } from "./common/base-dialog";
// babylon
import * as GUI from 'babylonjs-gui';
import { SingleArmyDialog } from "./army-dialog";

export class ArmiesDialog extends EmptyDialog {
    pos: WorldPos;
    armiesAtPos: Array<UnitArmy>;

    constructor(pos: WorldPos) {
        super(DIAG_ARMIES, true, "Armies");
        this.pos = pos;
        this.armiesAtPos = globalThis.ARMIES.getArmiesAtPos(pos);
    }

    public addCustomContent(): void {
        for (const army of this.armiesAtPos) {
            const keyStr = this.getArmyNameStr(army);
            const valueStr = `${army.units.length} Units`;
            const armyIcon = army.armyTypeIcon;
            this.addNewDataRow(keyStr, valueStr, armyIcon, true);
        }
    }

    private getArmyNameStr(army: UnitArmy): string {
        return `${army.armyTypeName} (${army.num})`;
    }

    private getArmyNumByKeyStr(datarowKeyStr: string): number {
        // cut off forst part
        let armyNumInvalid: number = -2;
        const armyNumFullStr = datarowKeyStr.split(" ")[1];
        const armyNumStr = armyNumFullStr.replace("(", "").replace(")", "");
        let armyNum: number = parseInt(armyNumStr);
        return armyNum;
    }

    /* GUI INTERACTION */
    onElementClicked(elemId: string): boolean {
        const success = super.onElementClicked(elemId);
        const armyRowKey = elemId.replace("datarow-", "");
        const armyNum = this.getArmyNumByKeyStr(armyRowKey);
        if (armyNum) {
            const army = globalThis.ARMIES.getArmyByNum(armyNum);
            // show single army dialog
            const singleArmyDiag = new SingleArmyDialog(army);
            this.close();
            globalThis.GUI.showDialog(singleArmyDiag);
            return true;
        }
        return false;
    }

    /* EVENT LISTENING */
    onEvent(ev: IBaseEvent) {
        super.onEvent(ev);
        // ELEMENT CLICKED
        if ( ev.id = EV.ID_GUI_ELEM_CLICKED) {
            let clickedElemId: string = ev.properties.getStr("elemId");
            if (clickedElemId) { this.onElementClicked( clickedElemId ); }   
        }
    }
}

export const DIAG_ARMIES = "diag-armies";