// entities
import { City} from "../../entities/city/city";
import { Building } from "../../entities/building/building";
// gui
import { EmptyDialog } from "./common/base-dialog";
import { CityConstructionDialog } from "./city-construction-dialog";
import {BuildingRowElement} from "../elements/entities/building-row-element";

/* == CITY BUILDINGS DIALOG == */
export class CityBuildingsDialog extends EmptyDialog {
    city: City;

    constructor(city: City) {
        super(DIAG_CITYBLDS, false, `${city.name} Buildings`);
        this.city = city;
    }

    addCustomContent(): void {
        for (const building of this.city.buildings) {
            const elemId: string = `bldrow-${building.bldType.id}`;
            const bldElem: BuildingRowElement = new BuildingRowElement(elemId, building);
            this.addControl(bldElem, true);
            // add row for building in city TODO: replace with BuildingRowElement
            // this.addNewDataRow(building.name, "" + building.level, "building");
        }
        this.addNewDivider();
        this.addNewButton(DIAG_CITYBLDS_BTN_CONSTRUCT, "Construct");
    }

    get buildings(): Array<Building> {
        return this.city.buildings;
    }

    onButtonClick(btnId: string): void {
        const success = super.onButtonClick(btnId);
        const city = this.city;
        if (btnId == DIAG_CITYBLDS_BTN_CONSTRUCT) {
            this.log('show City Construction Dialog');
            this.close();
            const diagCityConstr = new CityConstructionDialog(city);
            globalThis.GUI.showDialog(diagCityConstr, true);
        }
    }
}



/* == CONSTANTS == */
export const DIAG_CITYBLDS = "diag-city-buildings";
export const DIAG_CITYBLDS_ID_PREFIX = DIAG_CITYBLDS + "-";
export const DIAG_CITYBLDS_BTN_CONSTRUCT = DIAG_CITYBLDS_ID_PREFIX + "btn-construct";
