// events
import { IBaseEvent, BaseEvent, BaseEventListener } from "../../../common/events";
import * as EV from "../../game-events";
// entities
import { City } from "../../entities/city/city";
import { CityPopulation, POPS_UPDATE_GROWTH } from "../../entities/city/population";
import { SinglePopulationClass } from '../../entities/city/population-class';
// gui
import * as GUI_CONST from '../common/constants';
import { EmptyDialog, IDialog } from "./common/base-dialog";
import { CityConsumptionDialog } from "./city-consumption-dialog";

/* == CITY POP DIALOG == */
/* opened on city population row click; shows the class composition and resource requirements of this city */
export class CityPopulationDialog extends EmptyDialog {
    city: City;
    population: CityPopulation = null;

    constructor(city: City) {
        super(DIAG_POPULATION, false, `${city.name} Population`);
        this.city = city;
        this.population = city.population;
    }

    /* ADD CHILD ELEMENTS */
    addCustomContent(): void {
        // class pops
        this.addPopClassesList();
        // consumption
        const resDeliveredRowId = DIAG_POPULATION_ROW_CONSUMPTION_ID;
        const consumptionPercent = this.population.lastConsumptionSuccess * 100;
        const consumptionSuccessStr = `${consumptionPercent.toFixed(1)}%`
        this.addNewDataRow("Delivered", consumptionSuccessStr, "happiness", true, resDeliveredRowId);
        // total growth
        const growthStr = this.city.growth.toFixed(1) + "/" + POPS_UPDATE_GROWTH;
        this.addNewDataRow("Growth", growthStr, "growth");
        // last turn taxes
        this.addNewDataRow("Taxes", "" + this.city.taxesLastTurn, "money");
        // show "Talk To Mayor" button
        this.addNewButton(DIAG_POPULATION_BTN_TALKTOMAYOR, "Talk To Mayor");
    }

    addPopClassesList() {
        for (const cityClass of this.population.classes) {
            this.addSinglePopClassRow(cityClass);
        };
        // add divider after pop class list
        this.addNewDivider();
    }

    private addSinglePopClassRow(cityClass: SinglePopulationClass) {
        const classIcon = cityClass.iconName;
        const classPopsAmount = this.population.getAmount(cityClass.level);
        this.addNewDataRow(cityClass.name, `${classPopsAmount}`, classIcon);
    }

    /* EVENT HANDLING */
    onEvent(ev: IBaseEvent): void {
        super.onEvent(ev);
    }

    onElementClicked(elemId: string): boolean {
        const success = super.onElementClicked(elemId);
        const city = this.city;
        if(elemId.startsWith(GUI_CONST.ELEM_PREFIX_DATAROW)) {
            const datarowKey = elemId.replace(GUI_CONST.ELEM_PREFIX_DATAROW, "");
            if (datarowKey.startsWith("Delivered")) {
                const consumptionDiag = new CityConsumptionDialog(city);
                globalThis.GUI.showDialog(consumptionDiag);
                return true;
            }
        }
        return false;
    }
}

/* == CONSTANTS == */
export const DIAG_POPULATION = "diag-population";
export const DIAG_POPULATION_ROW_CONSUMPTION_ID = DIAG_POPULATION + "-datarow-Delivered";
export const DIAG_POPULATION_BTN_TALKTOMAYOR = DIAG_POPULATION + "-btn-talktomayor";
