// entities
import { Player } from "../../entities/player/player";
import { UnitArmy } from "../../entities/unit/army/army";
// trade
import { StoredResource } from "../../entities/trade/resource/resource-storage";
// gui
import * as GUI_CONST from '../common/constants';
import { EmptyDialog, IDialog } from "./common/base-dialog";
import { SingleArmyDialog } from "./caravan-units-dialog";
// babylon
import * as GUI from 'babylonjs-gui';

/* shows info about the player's caravan once clicked */
export class PlayerInfoDialog extends EmptyDialog {
  playerArmy: UnitArmy;

  constructor() {
    super(DIAG_PLAYER_INFO, false, "Player");
    this.playerArmy = globalThis.PLAYER.army;
  }

  build() {
    super.build();
  }

  addCustomContent(): void {
    // add labels
    this.addNewDataRow("family", PLAYER.family.name);
    // show stored resources
    this.showResourceInventory();
    // add buttons
    this.addNewButton(DIAG_PLAYER_BTN_INVENTORY, "Inventory");
    this.addNewButton(DIAG_PLAYER_BTN_UNITS, "Units");
  }

  private showResourceInventory() {
    const storage = globalThis.PLAYER.army.storedResources;
    for (const storedRes of storage) {
      this.showStoredResRow(storedRes);
    }
  }

  private showStoredResRow(storedRes: StoredResource) {
    let resName = globalThis.TRADE.getResName(storedRes.resId)
    let resPrice: number = storedRes.price;
    const elemId = `res-inv-${storedRes.resId}`;
    this.addNewDataRow(resName, resPrice + "o", "storage", false, elemId);
  }

  /* EVENT HANDLING */
  onButtonClick(btnId: string) {
    if (btnId === DIAG_PLAYER_BTN_INVENTORY) {
      this.log("show inventory diag");
    } else if (btnId === DIAG_PLAYER_BTN_UNITS) {
      // show player ArmyDialog
      const armyDiag = new SingleArmyDialog(this.playerArmy);
      globalThis.GUI.showDialog(armyDiag);
    }
  }

}

export const DIAG_PLAYER_INFO = "diag-player-info";
export const DIAG_PLAYER_BTN_INVENTORY = "diag-player-btn-inventory";
export const DIAG_PLAYER_BTN_UNITS = "diag-player-btn-units";
