// events
import { IBaseEvent, BaseEventListener } from '../../common/events';
import * as EV from '../game-events';
// trade
import { TradeConnection } from "../entities/trade/connection/connection";
import { IConnectedLocation, ILocation } from "../world/location";
// managers
import { AbstractManager } from "./abstract-manager";


export class ConnectionManager extends AbstractManager {
    connections: Array<TradeConnection> = [];

    constructor() {
        super("Connection");
    }

    makeCityConnections() {
        if (! globalThis.CITIES) { 
            this.warn(`cant create connections; CityManager not found`); return;
        }
        this.log("creating city connections ..");
        const createdConnections: Array<TradeConnection> = [];
        for ( const city of globalThis.CITIES.getAll() ) {
            const cityConnections =  city.addConnections( TradeConnection.MAX_DISTANCE );
            createdConnections.concat(cityConnections);
        }
        // add city connections to manager connection list
        this.connections.concat(createdConnections);
    }

    fromTo(source: IConnectedLocation, target: IConnectedLocation): TradeConnection {
        for (const conn of source.connections) {
            if ( conn.hasTarget(target) ) { return conn; }
        }
        return;
    }

    getMirrored(connection: TradeConnection): TradeConnection {
        const mirrored = this.fromTo(connection.target, connection.source);
        return mirrored;
    }
}