// common
import { Loggable } from "../../common/log";
// world
import { WorldTile } from "../world/tile/tile";
// models
import { HexDirection, WorldPos } from "../world/pos";
import { City } from "../entities/city/city";
// gui
import { CityDialog } from "../gui/dialogs/city-dialog";
// babylon
import * as BABYLON from 'babylonjs';
import { PlayerInfoDialog } from "../gui/dialogs/player-dialog";
// manager
import { AbstractManager } from "./abstract-manager";

const moveKeyNums = {
    37: new HexDirection(HexDirection.WEST), 39: new HexDirection(HexDirection.EAST),
    36: new HexDirection(HexDirection.NORTHWEST), 33: new HexDirection(HexDirection.NORTHEAST),
    35: new HexDirection(HexDirection.SOUTHWEST), 34: new HexDirection(HexDirection.SOUTHEAST)
}

const moveKeyCodes = {
    "A": new HexDirection(HexDirection.WEST), "D": new HexDirection(HexDirection.EAST),
    "W": new HexDirection(HexDirection.NORTHWEST), "E": new HexDirection(HexDirection.NORTHEAST),
    "Z": new HexDirection(HexDirection.SOUTHWEST), "X": new HexDirection(HexDirection.SOUTHEAST)
}

const KEY_NEXT_TURN_BTN = "N";
const KEY_TOGGLE_INSPECTOR = "I";

export class InputManager extends AbstractManager {
    private showingInspector: boolean = false;

    constructor() {
        super("Input");
    }

    movePlayer(keyCode: string): boolean {
        const dir = moveKeyCodes[keyCode] as HexDirection;
        const dirStr = dir.dirStr;
        globalThis.PLAYER.moveInDir(dirStr);
        return true;
    }

    private endTurn(): boolean {
        if (! globalThis.TURNS) {
            this.error("turn manager not loaded");
            return false;
        }
        globalThis.TURNS.onEndTurnButton();
        return true;
    }

    onKeyDown() {

    }

    onKeyUp(keyCode: string): boolean {
        if (! globalThis.MAP || ! globalThis.PLAYER) {
            this.error("map not loaded");
            return false;
        }
        // move player
        if (keyCode in moveKeyCodes) {
            return this.movePlayer(keyCode);
        // next turn
        } else if (keyCode == KEY_NEXT_TURN_BTN) {
            return this.endTurn();
        // toggle inspector
        } else if (keyCode == KEY_TOGGLE_INSPECTOR) {
            return this.toggleInspector();
        }
    }

    onMouseClick(event: PointerEvent, pick?: BABYLON.PickingInfo) {
        // this.log(`${event} ${pick.pickedMesh.id}`);
        if (pick.pickedMesh) {
            this.onMeshClicked(pick.pickedMesh);
        }
    }

    onMeshClicked(mesh: BABYLON.AbstractMesh) {
        if (globalThis.SELECTION) {
            globalThis.SELECTION.onMeshClicked(mesh);
        }
    }

    private toggleInspector(): boolean {
        this.showingInspector = ! this.showingInspector;
        if (this.showingInspector) {
            globalThis.SCENE.debugLayer.show();
        } else {
            globalThis.SCENE.debugLayer.hide();
        }
        return true;
    }

    toString(): string {
        return "<Input>";
    }
}