// events
import { BaseEventListener, IBaseEvent } from "../../common/events";
import * as EV from "../game-events";
// world
import { WorldPos } from "../world/pos";
import { WorldTools } from "../../helpers/world-tools";
// manager
import { AbstractManager } from "./abstract-manager";
// gui
import { PlayerInfoDialog } from "../gui/dialogs/player-dialog";
import { CityDialog } from "../gui/dialogs/city-dialog";
// babylon
import * as BABYLON from 'babylonjs';

export class PlayerManager extends AbstractManager {

    constructor() {
        super("Player");
    }

    movePlayerTo(newPos: WorldPos) {
        const playerStartPos = globalThis.PLAYER.pos;
        globalThis.PLAYER.setPos(newPos, null, playerStartPos);
    }

    getPlayerCursorSceneDistance(): number {
        const playerMesh = globalThis.PLAYER.mesh;
        const cursorMesh = globalThis.SELECTION.cursorMesh;
        return WorldTools.getMeshDistance(playerMesh, cursorMesh);
    }

    /* INTERNAL IN EVENT */
    onPlayerClicked() {
        this.log(`clicked on player`);
        const playerDiag = new PlayerInfoDialog();
        globalThis.GUI.showDialog(playerDiag);
    }

    beforePlayerMovement(targetPos: WorldPos) {
        globalThis.SELECTION.setSourceToPlayer();
    }

    beforeValidPlayerMovement(targetPos: WorldPos) {

    }

    canMoveTo(targetPos: WorldPos): boolean {
        const playerPos = globalThis.PLAYER.pos;
        const sceneDist = WorldTools.getWorldPosDistance(targetPos, playerPos);
        this.log(`dist to player target: ${sceneDist}`);
        return true;
    }

    afterPlayerMovement() {
        globalThis.PLAYER.updatePlayerYPos();
        // 4. city entered or left?
        globalThis.PLAYER.checkCityEnterLeave();
        globalThis.TURNS.onEndTurnButton();
    }

    checkOnEntered() {
      
    }

    onEvent(ev: IBaseEvent): void {

    }
}
