// events
import { BaseEventListener } from "../../common/events";
import * as EV from "../game-events";
// models
import { Turn } from "../../common/turn";
// manager
import { AbstractManager } from "./abstract-manager";

export class TurnManager extends AbstractManager {
    public pastTurns: Array<Turn> = [];
    public active: Turn = null;

    constructor() { 
        super("Turn");
        this.active = new Turn(0);
    }

    turnNum(): number {
        return this.active.num;
    }

    public onEndTurnButton(): void {
        // this.log(`End Turn button clicked`);
        // processing & fire turn finished event
        this.active.nextStage();
        globalThis.EVENTS.fireEvent( new EV.PlayerTurnFinishedEvent(this.active.num));
        // put after turn processing here
        globalThis.CITIES._onTurnFinished();
        // finishing
        this.active.nextStage();
    }

    public onTurnFinished(turn: Turn): void {
        // finishing
        this.startNextTurn();
    }

    public startNextTurn(): void {
        const oldTurn: Turn = this.active;
        const nextTurnNum = oldTurn.num + 1;
        this.pastTurns.push(oldTurn);
        this.active = new Turn(nextTurnNum);
        // fire before turn started event
        globalThis.EVENTS.fireEvent( new EV.BeforeTurnStartedEvent(this.active.num) );
        // fire turn started Event
        globalThis.EVENTS.fireEvent( new EV.TurnStartedEvent(this.active.num) );
        this.log(`started turn ${this.active}`);
    }
}