// common
import { RandomElement } from "../../helpers/random";
// events
import { IBaseEvent, BaseEventListener } from '../../common/events';
import * as EV from "../game-events";
// world
import { WorldPos } from "../world/pos";
// entities
import { Unit } from "../entities/unit/unit";
import { UnitTypes } from "../entities/unit/type/unit-types";
import { UnitType } from "../entities/unit/type/unit-type";
// manager
import { AbstractManager } from "./abstract-manager";

export class UnitManager extends AbstractManager {
    // lastArmyNum: number = -1;
    units: Array<Unit> = [];

    constructor() {
        super("Unit");
        // this.initializeUnitTypes();
    }

    createUnit(typeId: string, pos: WorldPos): Unit {
        const instance = new Unit(typeId, pos);
        this.units.push(instance);
        this.onUnitSpawned(instance);
        return instance;
    }

    /* UNIT TYPES */
    initializeUnitTypes() {
        this.log(`start unit type initialization`);
        globalThis.UNIT_TYPES = new UnitTypes();
        // fire event
        const typesLoadedEvent = new EV.UnitTypesInitializedEvent(globalThis.UNIT_TYPES.getAll().length);
        globalThis.EVENTS.fireEvent(typesLoadedEvent);
        // inform faction manager
        globalThis.FACTIONS.onUnitTypesInitialized();
    }

    isValidUnitTypeId(unitTypeId: string): boolean {
        if(! globalThis.UNIT_TYPES) {
            this.warn(`cant check whether "${unitTypeId}" is valid; UnitType list not found`);
            return false;
        }
        return globalThis.UNIT_TYPES.isValidUnitTypeId(unitTypeId);
    }

    getUnitTypeById(unitTypeId: string): UnitType {
        return globalThis.UNIT_TYPES.get(unitTypeId);
    }

    /* UNIT AND ARMY NUMBERS */
    nextUnitId(): number {
        return this.units.length;
    }

    /* ON EVENTS */
    onEvent(ev: IBaseEvent): boolean {
        super.onEvent(ev);
        return true;
    }

    onUnitSpawned(newUnit: Unit) {

    }

    onCitiesLoaded() {
        // globalThis.ARMIES.createStartingArmies();
    }

    onGearTypesLoaded() {
        this.initializeUnitTypes();
    }
}