// world
import { WorldTile } from "./tile";
import { WorldPos } from "../pos";

/* INTERFACES */
export interface IPathNode {
    parent: IPathNode;
    pos: WorldPos;
    findNext(): IPathNode;
}

export interface ITilePathNode extends IPathNode {
    tile: WorldTile;
}

export interface ITilePath {
    start: ITilePathNode;
    target: ITilePathNode;
    isFinished: boolean;
}

/* NODE */
export class TilePathNode implements ITilePathNode {
    tile: WorldTile;
    pos: WorldPos;
    parent: IPathNode = null;
    next: IPathNode = null;

    constructor(tile: WorldTile, parent?: IPathNode) {
        this.tile = tile;
        this.pos = tile.pos;
        if (parent) {
            this.parent = parent;
        }
    }

    add(child: TilePathNode) {
        this.next = child;
        child.parent = this;
    }

    findNext(): TilePathNode {
        return null;
    }
}

/* PATH */
export class TilePath implements ITilePath {
    start: ITilePathNode;
    target: ITilePathNode = null;
    isFinished: boolean = false;

    constructor(startTile: WorldTile, targetTile?: WorldTile) {
        this.start = new TilePathNode(startTile);
        if (targetTile) {
            this.target = new TilePathNode(targetTile);
        }
    }
}